include("global/noxiousdb.lua")

include("shared.lua")

include("sv_obj_player_extend.lua")

AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")

AddCSLuaFile("sh_micro.lua")

AddCSLuaFile("obj_entity_extend.lua")
AddCSLuaFile("sh_obj_player_extend.lua")

AddCSLuaFile("sh_animations.lua")

AddCSLuaFile("vgui/praceboard.lua")

GM.MapEditorPrefix = "hovergear"
include("mapeditor.lua")

if file.Exists("../gamemodes/hovergear/gamemode/maps/"..game.GetMap()..".lua") then
	MsgN("Executing map profile...")
	include("maps/"..game.GetMap()..".lua")
	MsgN("Done.")
end

function GM:Initialize()
	resource.AddFile("materials/noxctf/sprite_flame.vmt")
	resource.AddFile("materials/noxctf/sprite_smoke.vmt")
end

function GM:InitPostEntity()
	physenv.SetGravity(Vector(0, 0, SCALE * -800))
	RunConsoleCommand("sv_gravity", SCALE * 800)

	self:LoadMapEditorFile()

	local crowdspawns = {}
	local racingspawns = {}

	for _, oldspawn in pairs(ents.FindByClass("gmod_player_start")) do
		if oldspawn.RedTeam then
			table.insert(crowdspawns, oldspawn)
		else
			table.insert(racingspawns, oldspawn)
		end
	end

	table.Add(crowdspawns, ents.FindByClass("info_player_crowd"))
	table.Add(crowdspawns, ents.FindByClass("info_player_rebel"))
	table.Add(racingspawns, ents.FindByClass("info_player_racer"))
	table.Add(racingspawns, ents.FindByClass("info_player_combine"))

	if #crowdspawns == 0 then
		table.Add(crowdspawns, ents.FindByClass("info_player_*"))
		ErrorNoHalt(team.GetName(TEAM_CROWD).." team had no spawns. Using every spawn on the map.\n")
	end
	if #racingspawns == 0 then
		table.Add(racingspawns, ents.FindByClass("info_player_*"))
		ErrorNoHalt(team.GetName(TEAM_RACING).." team had no spawns. Using every spawn on the map.\n")
	end

	team.SetSpawnPoint(TEAM_CROWD, crowdspawns)
	team.SetSpawnPoint(TEAM_RACING, racingspawns)
end

local LastSpawnPoints = {}
function GM:PlayerSelectSpawn(pl)
	local teamid = pl:Team()
	local tab = team.GetSpawnPoint(teamid)
	local count = #tab
	if count == 0 then return pl end

	local potential = {}

	for _, spawn in pairs(tab) do
		if spawn:IsValid() and not spawn.Disabled and spawn ~= LastSpawnPoints[teamid] and spawn:IsInWorld() then
			local blocked
			for _, ent in pairs(ents.FindInBox(spawn:GetPos() + Vector(-20, -20, 0), spawn:GetPos() + Vector(20, 20, 74))) do
				if ent:IsPlayer() or ent:IsNPC() or string.sub(ent:GetClass(), 1, 5) == "prop_" then
					blocked = true
					break
				end
			end
			if not blocked then
				potential[#potential + 1 ] = spawn
			end
		end
	end

	if #potential > 0 then
		local spawn = potential[math.random(1, #potential)]
		LastSpawnPoints[teamid] = spawn
		return spawn
	end

	return LastSpawnPoints[teamid] or pl
end

function GM:PlayerChangedTeam(pl, oldteamid, newteamid)
	pl:SetLap(0)
	pl:SetCheckpoint(0)

	if newteamid == TEAM_RACING then
		pl.CheckpointPositions = {}
	else
		pl.CheckpointPositions = nil
	end
end

function GM:StartRace()
	if self.Racing then return end

	for _, ent in pairs(ents.FindByClass(self.VehicleClass)) do
		ent:Remove()
	end

	self.Racing = true
	self.StartRaceTime = CurTime()
	self.EndRaceTime = CurTime() + self.RaceTime

	gamemode.Call("UpdateGameState")

	for i=1, self.MaximumRacers do
		local pl = self.Queue[1]
		table.remove(self.Queue, 1)

		if pl then
			if pl:IsValid() then
				pl:StripWeapons()
				pl:SetTeam(TEAM_RACING)
				pl:Spawn()
			else
				i = i - 1
			end
		end
	end

	gamemode.Call("UpdateQueue")

	PrintMessage(HUD_PRINTCENTER, "The race has started!")
end

function GM:EndRace(winners)
	if not self.Racing then return end

	self.Racing = nil
	self.EndRaceTime = nil
	self.StartRaceTime = nil

	gamemode.Call("UpdateGameState")

	PrintMessage(HUD_PRINTCENTER, "The race has ended!")

	if #team.GetPlayers(TEAM_RACING) == 1 then
		winners = nil
	end

	if winners then
		if type(winners) == "table" then
			if #winners == 0 then
				PrintMessage(HUD_PRINTTALK, "No one won the race!")
			else
				for i, winner in ipairs(winners) do
					PrintMessage(HUD_PRINTTALK, winner:Name().." has won the race!")
				end
			end
		else
			PrintMessage(HUD_PRINTTALK, winners:Name().." has won the race!")
		end
	else
		PrintMessage(HUD_PRINTTALK, "No one won the race!")
	end

	for _, pl in pairs(team.GetPlayers(TEAM_RACING)) do
		pl:StripWeapons()
		pl:ExitVehicle()
		pl:SetTeam(TEAM_CROWD)
		pl:Spawn()
	end

	for _, ent in pairs(ents.FindByClass(self.VehicleClass)) do
		ent:Remove()
	end
end

function GM:ShouldRaceEnd()
	if #team.GetPlayers(TEAM_RACING) < self.MinimumRacers or self.EndRaceTime <= CurTime() then
		return true
	end
end

function GM:PlayerDeathThink(pl)
	if pl.NextSpawnTime <= CurTime() and (pl:Team() == TEAM_RACING or pl:KeyPressed(IN_ATTACK) or pl:KeyPressed(IN_ATTACK2) or pl:KeyPressed(IN_JUMP)) then
		pl:Spawn()
	end
end

function GM:Think()
	if self.Racing then
		if self.EndRaceTime <= CurTime() then
			gamemode.Call("EndRace")
		end
	else
		if self.MinimumRacers <= #self.Queue then
			if self.StartRaceTime then
				if self.StartRaceTime <= CurTime() then
					gamemode.Call("StartRace")
				end
			else
				self.StartRaceTime = CurTime() + self.PreRaceCountdown
				gmod.BroadcastLua("GAMEMODE.StartRaceTime="..tostring(self.StartRaceTime))
			end
		elseif self.StartRaceTime then
			self.StartRaceTime = nil
			gmod.BroadcastLua("GAMEMODE.StartRaceTime=nil")
		end
	end
end

function GM:PlayerDisconnect(pl)
	if pl:IsValid() then
		pl:RemoveHoverGear()
		pl:ExitQueue(true)
	end

	if self:ShouldRaceEnd() then
		gamemode.Call("EndRace", team.GetPlayers(TEAM_RACING))
	end
end

function GM:UpdateGameState(filter)
	umsg.Start("updgmstate", filter)
		umsg.Bool(self.Racing or false)
		umsg.Float(self.StartRaceTime or 0)
		umsg.Float(self.EndRaceTime or 0)
	umsg.End()
end

function GM:PlayerReady(pl)
	gamemode.Call("UpdateGameState", pl)

	pl:SendLua("GAMEMODE.MaxCheckpoint="..tostring(GAMEMODE.MaxCheckpoint))
end

function GM:PlayerInitialSpawn(pl)
	pl.NextSpawnTime = 0

	pl:SetTeam(TEAM_NOTRACING)
end

function GM:PlayerHitCheckpoint(pl, point)
	if point == pl:Deaths() + 1 then
		pl:PrintMessage(HUD_PRINTCENTER, "You hit a checkpoint!")
		pl:SendLua("surface.PlaySound(\"buttons/button14.wav\")")
		pl:SetDeaths(point)

		if pl:InVehicle() then
			pl.CheckpointPositions[point] = {pl:GetVehicle().VehicleParent:GetPos(), pl:GetVehicle().VehicleParent:GetAngles()}
		else
			pl.CheckpointPositions[point] = {pl:GetPos(), pl:GetAngles()}
		end
	elseif point == 0 and pl:Deaths() == self.MaxCheckpoint then
		if pl:InVehicle() then
			pl.CheckpointPositions = {{pl:GetVehicle().VehicleParent:GetPos(), pl:GetVehicle().VehicleParent:GetAngles()}}
		else
			pl.CheckpointPositions = {}
		end

		PrintMessage(HUD_PRINTCENTER, pl:Name().." finished a lap!")
		pl:AddFrags(1)
		pl:SetDeaths(0)
		if pl:Frags() == self.LapsToWin then
			gamemode.Call("EndRace", pl)
		else
			pl:SendLua("surface.PlaySound(\"buttons/button19.wav\")")
		end
	elseif point ~= pl:Deaths() then
		pl:PrintMessage(HUD_PRINTCENTER, "You're going the wrong way!!")
	end
end

function GM:GravGunPunt(pl, ent)
	return string.sub(ent:GetClass(), 1, 8) == "vehicle_" and (not ent.Owner or ent.Owner == pl)
end

function GM:PlayerSpawn(pl)
	pl:SetHull(PLAYER_HULL_MIN, PLAYER_HULL_MAX)
	pl:SetHullDuck(PLAYER_HULL_MIN, PLAYER_HULL_MAX)
	pl:SetCollisionBounds(PLAYER_HULL_MIN, PLAYER_HULL_MAX)
	pl:SetViewOffset(PLAYER_VIEWOFFSET)
	pl:SetViewOffsetDucked(PLAYER_VIEWOFFSET_DUCKED)
	pl:SetStepSize(PLAYER_STEPSIZE)
	pl:SetJumpPower(PLAYER_JUMPPOWER)

	pl:PhysicsInitBox(PLAYER_HULL_MIN, PLAYER_HULL_MAX)
	pl:SetSolid(SOLID_BBOX)
	pl:SetMoveType(MOVETYPE_WALK)

	GAMEMODE:SetPlayerSpeed(pl, PLAYER_WALKSPEED, PLAYER_RUNSPEED)

	pl:SetModel(player_manager.TranslatePlayerModel(pl:GetInfo("cl_playermodel")))

	if pl:Team() == TEAM_RACING then
		local point = pl:Deaths()
		local checktab = pl.CheckpointPositions[point]
		if checktab then
			local hovergear = pl:SpawnHoverGear(checktab[1], checktab[2])
			if hovergear and hovergear:IsValid() then
				hovergear:SetVehicleHealth(1)
			end
		else
			pl:SpawnHoverGear()
		end
	else
		pl:UnSpectate()
	end

	umsg.Start("PlayerSpawn")
		umsg.Entity(pl)
	umsg.End()
end

function GM:PlayerNoClip(pl, on)
	if pl:IsAdmin() then
		if on then
			LogAction("[Admin CMD] <"..pl:SteamID().."> "..pl:Name().." TURNED ON NOCLIP")
		else
			LogAction("[Admin CMD] <"..pl:SteamID().."> "..pl:Name().." TURNED OFF NOCLIP")
		end
		return true
	end

	return false
end

function GM:DoPlayerDeath(pl, attacker, dmginfo)
	pl.NextSpawnTime = CurTime() + 3

	if pl:Team() == TEAM_RACING then
		pl:PrintMessage(HUD_PRINTTALK, "You have been put back at your last checkpoint.")

		if self:ShouldRaceEnd() then
			gamemode.Call("EndRace", team.GetPlayers(TEAM_RACING))
		end
	end
end

function GM:ShowTeam(pl)
	if pl:Team() ~= TEAM_RACING and not pl:ExitQueue() then
		pl:EnterQueue()
	end
end

function GM:QueueUpdated()
end

function GM:UpdateQueue(filter)
	local tosend = {}
	for i, pl in ipairs(self.Queue) do
		tosend[#tosend + 1] = pl:EntIndex()
	end

	umsg.Start("updateracingqueue", filter)
		umsg.String(table.concat(tosend, ","))
	umsg.End()

	gamemode.Call("QueueUpdated")
end

local vec0 = Vector(0,0,0)
function GM:SetPlayerAnimation(pl)
	if pl:InVehicle() then
		local seq = pl:SelectWeightedSequence(ACT_HL2MP_SIT_FIST)
		if pl:GetSequence() ~= seq then
			pl:ResetSequence(seq)
		end
	else
		local act
		if pl:IsOnGround() then
			local notmoving = vec0 == pl:GetVelocity()
			if pl:Crouching() then
				if notmoving then
					act = ACT_HL2MP_IDLE_CROUCH
				else
					act = ACT_HL2MP_WALK_CROUCH
				end
			elseif notmoving then
				act = ACT_HL2MP_IDLE
			else
				act = ACT_HL2MP_RUN
			end
		else
			act = ACT_HL2MP_JUMP
		end

		local seq = pl:SelectWeightedSequence(pl:Weapon_TranslateActivity(act))
		if seq == -1 then
			seq = pl:SelectWeightedSequence(ACT_HL2MP_JUMP_SLAM)
		end
		if pl:GetSequence() ~= seq then
			pl:ResetSequence(seq)
		end
	end
end
