function EFFECT:Think()
	if not self.Emitted and self.DieTime - CurTime() <= 0.25 then
		self.Emitted = true

		local pos = self.Pos
		local normal = self.Normal
		local emitter = self.Emitter
		for i=1, EFFECT_QUALITY * 10 + 40 do
			local heading = (VectorRand() + normal):Normalize()
			local startpos = pos + heading * 16

			local particle = emitter:Add("sprites/glow04_noz", startpos)
			particle:SetDieTime(math.Rand(0.5, 0.75))
			particle:SetStartAlpha(255)
			particle:SetEndAlpha(128)
			particle:SetStartSize(32)
			particle:SetEndSize(0)
			particle:SetVelocity(heading * math.Rand(1600, 2048))
			particle:SetAirResistance(400)
			particle:SetCollide(true)
			particle:SetBounce(0.95)
			particle:SetRoll(math.Rand(0, 360))
			particle:SetRollDelta(math.Rand(-30, 30))
		end
		emitter:Finish()
	end

	return CurTime() < self.DieTime
end

local EndFlash = 0
local cmtab = {["$pp_colour_brightness"] = 1, ["$pp_colour_contrast"] = 1, ["$pp_colour_colour"] = 1}
local function FlashColorModify()
	cmtab["$pp_colour_brightness"] = math.max(0, math.min(5, EndFlash - CurTime()))
	DrawColorModify(cmtab)

	if cmtab["$pp_colour_brightness"] <= 0 then
		MySelf:SetDSP(0)
		hook.Remove("RenderScreenspaceEffects", "FlashEffect")
	end
end

function EFFECT:Init(data)
	local normal = data:GetNormal() * -1
	local pos = data:GetOrigin()
	local teamid = data:GetScale()
	local owner = data:GetEntity()

	if owner == MySelf or teamid ~= MySelf:Team() then
		local eyepos = MySelf:EyePos()
		local aimvec = MySelf:GetAimVector()

		local dist = pos:Distance(eyepos)
		if dist < 400 then
			local power = (400 - dist) * 0.005
			power = power * aimvec:Distance((eyepos - pos):Normalize())

			EndFlash = CurTime() + power
			MySelf:DI(NameToSpell.Flash, power)
			MySelf:SetDSP(32)
			hook.Add("RenderScreenspaceEffects", "FlashEffect", FlashColorModify)
		end
	end

	self.DieTime = CurTime() + 0.5

	util.Decal("FadingScorch", pos - normal, pos + normal)

	WorldSound("nox/plasma_explode.wav", pos, 77, math.Rand(120, 128))

	self.Pos = pos
	self.Normal = normal
	self.Col = Color(255, 255, 255, 255)

	local emitter = ParticleEmitter(pos)
	emitter:SetNearClip(28, 36)
	self.Emitter = emitter
	for i=1, EFFECT_QUALITY * 10 + 40 do
		local heading = (VectorRand() * 2 + normal):Normalize()
		local startpos = pos + heading * 16

		local particle = emitter:Add("sprites/glow04_noz", startpos)
		particle:SetDieTime(math.Rand(1, 1.25))
		particle:SetStartAlpha(255)
		particle:SetEndAlpha(128)
		particle:SetStartSize(32)
		particle:SetEndSize(0)
		particle:SetVelocity(normal * 1000)
		particle:SetGravity(heading * 2000)
		particle:SetAirResistance(200)
		particle:SetCollide(true)
		particle:SetBounce(0.95)
		particle:SetRoll(math.Rand(0, 360))
	end
end

local matGlow = Material("sprites/glow04_noz")
function EFFECT:Render()
	local pos = self.Pos
	local normal = self.Normal

	local delta = self.DieTime - CurTime()

	local size
	if 0.25 < delta then
		size = delta * 1024
	else
		size = 1024 - delta * 1024
	end

	local col = self.Col
	col.a = delta * 510

	render.SetMaterial(matGlow)
	render.DrawQuadEasy(pos, normal, size, size, col)
	render.DrawQuadEasy(pos, normal, size, size, col)
	render.DrawQuadEasy(pos, normal * -1, size, size, col)
	render.DrawQuadEasy(pos, normal * -1, size, size, col)
	render.DrawSprite(pos, size, size, col)
	render.DrawSprite(pos, size, size, col)
end
