AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")

include("shared.lua")

function ENT:Initialize()
	self:SetModel("models/Roller_Spikes.mdl")
	self:PhysicsInit(SOLID_VPHYSICS)
	self:SetSolid(SOLID_VPHYSICS)
	self:SetCollisionGroup(COLLISION_GROUP_WEAPON)
	self:SetTrigger(true)

	self.Carrier = self
	self.NextCantTakeMsg = 0
	self.LastDrop = 0
	self.LastThink = CurTime()

	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		phys:SetMass(500)
		phys:EnableMotion(false)
		phys:Wake()
	end
end

function ENT:TouchedGoal(pl, goal)
	if pl:IsValid() and pl:IsPlayer() then
		local teamid = pl:Team()
		if teamid ~= goal:GetTeamID() then
			self:SetCarrier(self)
			self:SetPos(goal:GetPos() + Vector(0, 0, 16))
			GAMEMODE:BallScored(pl, teamid, self, goal)
		end
	end
end

function ENT:Dropped(pl)
	self:SetCarrier(self)

	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		phys:EnableMotion(true)
	end

	if pl and pl:IsValid() then
		self:SetPos(pl:GetPos() + Vector(0,0,16))
		pl.Carrying = nil
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetVelocity() * 0.75 + Vector(0,0,300))
		end
	end

	self.LastDrop = CurTime()
	self.AutoReturnTimer = CurTime() + 20

	self:SetSkin(2)

	GAMEMODE:BallDropped(pl)
end

function ENT:Reset(silent)
	self:SetToHome()
	if not silent then
		GAMEMODE:BallReset()
	end
end

function ENT:SetToHome()
	local newpos = GAMEMODE.BallDrop

	if 256 <= newpos:Distance(self:GetPos()) then
		umsg.Start("FlagReturnEffect")
			umsg.Vector(self:GetPos())
			umsg.Vector(newpos)
			umsg.Short(TEAM_SPECTATE)
		umsg.End()
	end

	self:SetCarrier(self)
	self:SetPos(newpos)
	self:SetSkin(0)

	self.AutoReturnTimer = nil

	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		phys:EnableMotion(false)
	end
end

function ENT:SetCarrier(ent)
	if ent == self then
		self:SetSolid(SOLID_VPHYSICS)
		self:SetOwner(NULL)
		self:SetColor(255, 255, 255, 255)

		self.Carrier = self

		local phys = self:GetPhysicsObject()
		if phys:IsValid() then
			phys:EnableGravity(true)
		end

		for _, pl in pairs(player.GetAll()) do
			if pl.Carrying == self then
				pl.Carrying = nil
				pl:SetStamina(pl:GetStamina(), true)
			end
		end
	elseif ent and ent:IsValid() then
		self:SetSolid(SOLID_NONE)
		self:SetOwner(ent)
		self:SetSkin(1)
		local col = team.GetColor(ent:GetTeamID())
		self:SetColor(col.r, col.g, col.b, 255)

		self.AutoReturnTimer = nil
		self.LastThink = CurTime()
		self.Carrier = ent

		ent.Carrying = self

		local phys = self:GetPhysicsObject()
		if phys:IsValid() then
			phys:EnableGravity(false)
			phys:EnableMotion(false)
			phys:Wake()
		end

		if ent.Invisible then
			DeInvisible(ent)
		end
	end
end

function ENT:Touch(hitent)
	if self.Carrier == self and hitent:IsPlayer() and not ENDGAME and GAMESTATE == GAMESTATE_PLAY and not GAMEMODE.Intermission and self.Carrier == self and not hitent:InVehicle() then
		if hitent.LastDeath + 0.25 <= CurTime() then
			if 0 < #GetOpposingPlayers(hitent:Team()) then
				if self.LastDrop + 0.5 <= CurTime() then
					self:SetCarrier(hitent)
					GAMEMODE:BallTaken(hitent)
				end
			elseif self.NextCantTakeMsg < CurTime() then
				hitent:PrintMessage(HUD_PRINTCENTER, "You can't take the ball when there's no one playing against you!")
				self.NextCantTakeMsg = CurTime() + 0.5
			end
		end
	end
end

function ENT:Think()
	local carrier = self.Carrier

	if carrier == self then
		if self.AutoReturnTimer and self.AutoReturnTimer <= CurTime() then
			self:Reset()
		end

		return
	end

	local delta = CurTime() - self.LastThink
	self.LastThink = CurTime()

	if carrier:IsValid() and carrier:Alive() then
		self:SetPos(carrier:GetPos() + Vector(0, 0, 90))
		self:GetPhysicsObject():SetVelocityInstantaneous(Vector(0, 0, 0))

		carrier:SetStamina(math.max(carrier:GetStamina() - (carrier.StaminaRegenerate + STAMINA_HOLDING_BALL) * delta, 0))

		self:NextThink(CurTime())
		return true
	elseif carrier:IsValid() then
		self:Dropped(carrier)
	else
		self:Dropped()
	end
end
