AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")

include("shared.lua")

function ENT:Initialize()
	self:PhysicsInit(SOLID_VPHYSICS)
	self:SetSolid(SOLID_VPHYSICS)

	self.DesiredCannonYaw = 0
	self.DesiredCannonPitch = 0

	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		phys:EnableMotion(false)
		phys:Wake()
	end

	self.NextShoot = 0

	self.AmmoType = self.AmmoType or "NONE"

	self.LastAttacked = 0
	self.LastDriver = NULL

	self.Attacker = NULL
	self.Inflictor = NULL
end

function ENT:Attach(turretconnecting, activator)
	if self.AmmoType == turretconnecting.AmmoType then
		if activator then
			activator:PrintMessage(HUD_PRINTCENTER, "Turret already loaded with that type of barrel.")
			activator:SendLua([[surface.PlaySound("buttons/button16.wav")]])
		end
		return
	end

	if 200 < turretconnecting:GetPos():Distance(self:GetPos()) then
		if activator then
			activator:PrintMessage(HUD_PRINTCENTER, "Barrel is too far from the turret to load.")
			activator:SendLua([[surface.PlaySound("buttons/button16.wav")]])
		end
		return
	end

	if self.Barrel and self.Barrel:IsValid() then
		self.Barrel:Remove()
		self.Barrel = nil
	end

	local ent = ents.Create("turretbarrel")
	if ent:IsValid() then
		self.AmmoType = turretconnecting.AmmoType
		ent:SetModel(turretconnecting:GetTurretModel(self))
		ent:SetPos(turretconnecting:GetTurretPos(self, self.PilotSeat))
		ent:SetAngles(turretconnecting:GetTurretAngles(self, self.PilotSeat))
		ent:SetOwner(self)
		ent:SetParent(self.PilotSeat)
		local col = team.GetColor(self:GetTeamID())
		ent:SetColor(col.r, col.g, col.b, 255)
		ent:SetSkin(turretconnecting.SkinType)
		ent:Spawn()
		self.Barrel = ent
		turretconnecting:BarrelChanged(self, ent)
		if activator then
			activator:PrintMessage(HUD_PRINTCENTER, "Turret barrel changed!")
			activator.mTurretConnecting = nil
		end

		turretconnecting.NoValue = true
		GAMEMODE:RemoveProp(NULL, turretconnecting)
	elseif activator then
		activator:PrintMessage(HUD_PRINTCENTER, "Failed to attach barrel (engine error).")
	end
end

function ENT:AcceptInput(name, activator, caller, args)
	if name == "created" then
		self:CreateChildren()
	end
end

function ENT:OnRemove()
end

function ENT:Exit(pl, veh, role)
	self.LastDriver = NULL
end

function ENT:Enter(pl, veh, role)
	self.LastDriver = pl
end

function ENT:CreateChildren()
	if self.CreatedChildren then return end
	self.CreatedChildren = true

	local vPos = self:GetPos()
	local vForward = self:GetForward()
	local vRight = self:GetRight()
	local vUp = self:GetUp()

	local aAngles = self:GetAngles()

	local ent = ents.Create("prop_vehicle_prisoner_pod")
	if ent:IsValid() then
		ent:SetKeyValue("model", "models/props_phx/carseat2.mdl")
		ent:SetKeyValue("solid", "6")
		ent:SetKeyValue("vehiclescript", "scripts/vehicles/prisoner_pod.txt")
		ent:SetPos(vPos + vUp * 62)
		ent:SetAngles(aAngles)
		ent:SetMaterial("models/debug/debugwhite")
		local col = team.GetColor(self:GetTeamID())
		ent:SetColor(col.r, col.g, col.b, 255)
		ent:Spawn()
		ent:SetVehicle(self)

		constraint.Axis(ent, self, 0, 0, Vector(0, 0, -16), Vector(0, 0, 21), 0, 0, 0, 1)

		self:DeleteOnRemove(ent)
		self.PilotSeat = ent
		ent:SetNetworkedEntity("vehicle", self)
		self:SetNetworkedEntity("driverseat", ent)
		ent.Protects = true
		ent.Turret = true
	end

	local Seat = ent

	local ent = ents.Create("mturretcannon")
	if ent:IsValid() then
		ent:SetPos(vPos + vUp * 120)
		local tang = aAngles
		tang:RotateAroundAxis(tang:Forward(), -90)
		ent:SetAngles(tang)
		local col = team.GetColor(self:GetTeamID())
		ent:SetColor(col.r, col.g, col.b, 255)
		ent:SetOwner(Seat)
		ent:SetParent(Seat)
		ent:Spawn()
		ent:SetVehicle(self)
		self.Cannon = ent
		self:DeleteOnRemove(ent)
		ent:SetNetworkedEntity("vehicle", self)
		self:SetNetworkedEntity("cannon", ent)
	end
end

function ENT:Use(activator, caller)
	if self.Destroyed then return end
	if not activator:IsPlayer() then return end

	if activator:Team() == self:GetTeamID() then
		local turretconnecting = activator.mTurretConnecting
		if turretconnecting and turretconnecting:IsValid() then
			self:Attach(turretconnecting, activator)
		end
	end
end

function ENT:Think()
	if not self.CreatedChildren then
		if GAMESTATE == GAMESTATE_PLAY and not self.Destroyed then
			self:CreateChildren()
		end
		return
	end

	local driver = self.PilotSeat:GetDriver()

	if driver:IsValid() then
		if driver:KeyDown(IN_ATTACK) and self.NextShoot < CurTime() then
			if self["Attack_"..self.AmmoType] then
				self["Attack_"..self.AmmoType](self, driver, self.Cannon)
			else
				self.NextShoot = CurTime() + 0.5
				driver:PrintMessage(HUD_PRINTCENTER, "No mountable-turret barrel has been loaded!")
			end
		end

		if driver:KeyDown(IN_MOVELEFT) then
			local ang = self.PilotSeat:GetAngles()
			ang:RotateAroundAxis(ang:Up(), FrameTime() * 120)
			self.PilotSeat:SetAngles(ang)
		elseif driver:KeyDown(IN_MOVERIGHT) then
			local ang = self.PilotSeat:GetAngles()
			ang:RotateAroundAxis(ang:Up(), FrameTime() * -120)
			self.PilotSeat:SetAngles(ang)
		end

		self.PilotSeat:GetPhysicsObject():SetAngleDragCoefficient(1000000)

		self:NextThink(CurTime())
		return true
	else
		self.PilotSeat:GetPhysicsObject():SetAngleDragCoefficient(1)
	end
end

function ENT:Attack_Mortar(driver, cannon)
	self.NextShoot = CurTime() + 1.25

	if GAMEMODE:DrainPower(self, 15) then
		local ent = ents.Create("projectile_dropshipcannon")
		if ent:IsValid() then
			local aimvec = driver:GetAimVector()
			ent:SetPos(driver:GetShootPos() + aimvec * 32)
			ent:SetOwner(driver)
			ent:SetSkin(self:Team())
			ent:Spawn()
			ent.DisplayClass = "prop_mturretbarrel_mortar"
			ent:SetTeamID(driver:Team())
			cannon:EmitSound("vehicles/Airboat/pontoon_impact_hard1.wav", 80, math.Rand(95, 105))
			local phys = ent:GetPhysicsObject()
			if phys:IsValid() then
				phys:SetVelocityInstantaneous(aimvec * 1400)
			end
		end
	else
		driver:PrintMessage(HUD_PRINTCENTER, "Insufficient Mana. This turret barrel needs 15 Mana from a Nearby Capacitor to fire.")
	end
end

function ENT:Attack_Plasma(driver, cannon)
	self.NextShoot = CurTime() + 1.25

	if GAMEMODE:DrainPower(self, 10) then
		local proj = ents.Create("projectile_plasmabolt")
		if proj:IsValid() then
			proj:SetPos(cannon:GetPos() + driver:GetAimVector() * 48)
			local col = team.GetColor(self:GetTeamID())
			proj:SetColor(col.r,col.g,col.b,127)
			proj:Spawn()
			proj.DisplayClass = "projectile_plasma_mountablecannon"
			proj:SetTeamID(driver:Team())
			proj:SetOwner(driver)
			proj:GetPhysicsObject():ApplyForceCenter(driver:GetAimVector() * 4000)
		end
	else
		driver:PrintMessage(HUD_PRINTCENTER, "Insufficient Mana. This turret barrel needs 10 Mana from a nearby Mana Capacitor to fire.")
	end
end

function ENT:Attack_Lockon(driver, cannon)
	local fwd = driver:GetAimVector()
	local startpos = cannon:GetPos() + fwd * 64
	--self:FireBullets({Num = 1, Dir = fwd, Src = startpos + fwd * 16, Spread = Vector(0, 0, 0), Tracer = 0, Force = 0, Damage = 0, HullSize = 48, Callback = LockOnCB})

	local hitent = util.TraceHull({start = startpos, endpos = startpos + fwd * 16000, mask = MASK_SHOT, filter = {self, self.PilotSeat}, mins = Vector(-24, -24, -24), maxs=Vector(24, 24, 24)}).Entity or NULL

	local ent = NULL
	if hitent:IsValid() then
		if hitent.ScriptVehicle and hitent:Team() ~= self:GetTeamID() then
			ent = hitent
		elseif hitent.GetVehicle then
			local veh = hitent:GetVehicle()
			if veh:IsValid() and veh:GetTeamID() ~= self:GetTeamID() then
				ent = veh
			end
		end
	end

	if not ent:IsValid() then
		driver:PrintMessage(HUD_PRINTCENTER, "No enemy vehicles in crosshair!")
		self.NextShoot = CurTime() + 0.075
		return
	end

	self.NextShoot = CurTime() + 4

	if GAMEMODE:DrainPower(self, 30) then
		local proj = ents.Create("projectile_ravenmissile")
		if proj:IsValid() then
			cannon:EmitSound("weapons/rpg/rocketfire1.wav")
			local col = team.GetColor(driver:Team())
			proj:SetColor(col.r, col.g, col.b, 255)
			proj:SetPos(startpos)
			proj:Spawn()
			proj.DisplayClass = "prop_mturretbarrel_lockons"
			proj:SetTeamID(driver:Team())
			proj.Target = ent

			driver:SendLua("surface.PlaySound(\"buttons/button3.wav\")")

			for _, veh in pairs(ents.FindByClass("prop_vehicle_prisoner_pod")) do
				if veh:GetNetworkedEntity("vehicle") == ent then
					local enemydriver = veh:GetDriver()
					if enemydriver:IsPlayer() then enemydriver:SendLua("RLO()") end
				end
			end

			proj:SetOwner(driver)
			local phys = proj:GetPhysicsObject()
			if phys:IsValid() then
				phys:SetVelocityInstantaneous(fwd * 2800)
			end
		end
	else
		driver:PrintMessage(HUD_PRINTCENTER, "Insufficient Mana. This turret barrel needs 30 Mana from a nearby Mana Capacitor to fire.")
	end
end

function ENT:Info(pl)
	if pl:Team() == self:GetTeamID() then
		return self.PHealth..","..self.MaxPHealth..","..self.AmmoType
	end

	return "deny"
end

ENT.DestructionEffect = function(self, damage, attacker, inflictor)
	local effectdata = EffectData()
		effectdata:SetOrigin(self:GetPos())
	util.Effect("firebombexplosion", effectdata, true, true)

	local ps = self.PilotSeat
	if self.PilotSeat and self.PilotSeat:IsValid() then
		self.PilotSeat.Protects = nil
		if ps:GetDriver():IsValid() then
			ps:GetDriver():RemoveStatus("forcefield", false, true)
		end
	end

	ExplosiveDamage(attacker or self, self, self:GetPos(), 250, 250, 1, 0.6, 8)
end

ENT.MoneyBonus = 30
ENT.OffenseBonus = 3
ENT.FragsBonus = 3
ENT.ProcessDamage = PROPGENERICPROCESSDAMAGE
ENT.OverrideTakeDamage = PROPGENERICOVERRIDETAKEDAMAGE
ENT.ProcessSpecialDamage = PROPGENERICPROCESSSPECIALDAMAGE
