include("shared.lua")

function ENT:Initialize()
	self:DrawShadow(false)
	self:SetRenderBounds(Vector(-128, -128, -128), Vector(128, 128, 128))

	self:GetOwner().Lightning = self

	self:EmitSound(string.format("nox/lightning%02d.wav", math.random(1,2)), 72, math.Rand(95, 105))

	self.AmbientSound = CreateSound(self, "ambient/machines/electric_machine.wav")
	self.AmbientSound:PlayEx(0.72, 125)

	self.Emitter = ParticleEmitter(self:GetPos())
	self.Emitter:SetNearClip(20, 32)

	self.LastThink = CurTime()
end

function ENT:Think()
	local owner = self:GetOwner()
	if owner:IsValid() then self:SetPos(owner:EyePos()) end
	if MySelf == owner then
		owner:SetMana(owner:GetMana() - (CurTime() - self.LastThink) * 17)
		self.LastThink = CurTime()
	end
end

function ENT:OnRemove()
	self.AmbientSound:Stop()
	self.Emitter:Finish()
	self:GetOwner().Lightning = nil
end

local matBolt = Material("Effects/laser1")
local matGlow = Material("sprites/glow04_noz")
local function DrawBeamToFrom(to, from, rt, emitter)
	local BeamPos = {}
	local pos
	local dir = (to - from):Normalize()
	for i=1, math.ceil(from:Distance(to) / 40) do
		if pos then
			pos = pos + dir * 40 + VectorRand() * 4
		else
			pos = from
		end
		BeamPos[#BeamPos + 1] = pos
	end

	local scroll = rt * 20
	--render.StartBeam(#BeamPos * 2 + 2)
	render.StartBeam(#BeamPos * 2)
	for i, pos in ipairs(BeamPos) do
		render.AddBeam(pos, math.sin((rt + i * 0.2) * 6) * 24 + 48, scroll, COLOR_CYAN)
		render.AddBeam(pos, math.sin((rt + i * 0.4) * 6) * 24 + 48, scroll, color_white)
	end
	--render.AddBeam(to, 50, scroll, COLOR_CYAN)
	--render.AddBeam(to, 50, scroll, color_white)
	render.EndBeam()

	if EFFECT_QUALITY < 1 then return end

	local particle = emitter:Add("sprites/glow04_noz", to)
	particle:SetVelocity(Vector(0,0,0))
	particle:SetDieTime(0.22)
	particle:SetStartAlpha(255)
	particle:SetEndAlpha(127.5)
	particle:SetStartSize(math.Rand(28, 32))
	particle:SetEndSize(0)
	particle:SetRoll(math.Rand(0, 360))
	particle:SetRollDelta(math.Rand(-16, 16))

	for i=1, math.random(2, 4) do
		local particle = emitter:Add("effects/spark", to)
		particle:SetVelocity(VectorRand():Normalize() * math.Rand(64, 128))
		particle:SetDieTime(0.8)
		particle:SetStartAlpha(255)
		particle:SetEndAlpha(0)
		particle:SetStartSize(math.Rand(6, 8))
		particle:SetEndSize(0)
		particle:SetRoll(math.Rand(0, 360))
		particle:SetRollDelta(math.Rand(-75, 75))
		particle:SetAirResistance(50)
	end
end

function ENT:DrawTranslucent()
	local owner = self:GetOwner()
	if not owner:IsValid() then return end

	local tr = owner:TraceLine(1024, MASK_SHOT)
	local hitpos = tr.HitPos
	local ent = tr.Entity

	local startpos
	local wep = owner:GetActiveWeapon()
	if wep:IsValid() then
		local attach
		if owner == MySelf and not NOX_VIEW then
			attach = owner:GetViewModel():GetAttachment(1)
		else
			attach = wep:GetAttachment(1)
		end
		if attach then
			startpos = attach.Pos
		end
	end

	startpos = startpos or owner:GetShootPos()
	self:SetRenderBoundsWS(startpos, hitpos, Vector(2048, 2048, 2048))

	local myteam = owner:Team()
	local col = team.GetColor(myteam)
	local rt = RealTime()
	render.SetMaterial(matGlow)
	local spritesize = 52 + math.cos(rt * 10) * 14
	render.DrawSprite(startpos, spritesize, spritesize, color_white)
	render.DrawSprite(startpos, 52, 52, COLOR_CYAN)

	render.SetMaterial(matBolt)
	local emitter = self.Emitter

	local todamage = {}
	if ent and ent:IsValid() then
		if ent:IsPlayer() and ent:GetTeamID() ~= myteam then
			todamage[ent] = startpos

			while true do
				local startpos = ent:LocalToWorld(ent:OBBCenter())
				local found
				for _, e in pairs(ents.FindInSphere(startpos, 512)) do
					if not todamage[e] and e:IsPlayer() and e:Alive() and e:GetTeamID() ~= myteam and TrueVisible(startpos, e:NearestPoint(startpos)) then
						todamage[e] = ent
						ent = e
						found = true
						break
					end
				end
				if not found then break end
			end

			for e, targete in pairs(todamage) do
				local st = e:LocalToWorld(e:OBBCenter())
				if type(targete) == "Vector" then
					DrawBeamToFrom(st, targete, rt, emitter)
				else
					DrawBeamToFrom(st, targete:NearestPoint(st), rt, emitter)
				end
			end
		end
	end

	if table.Count(todamage) == 0 then
		DrawBeamToFrom(hitpos, startpos, rt, emitter)
	end
end
