AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")

include("shared.lua")

local EngineStart = Sound("ATV_engine_start")
local EngineIdleSlow = Sound("ATV_throttleoff_slowspeed")
local EngineIdle = Sound("ATV_throttleoff_fastspeed")
local EngineForward = Sound("ATV_reverse")
local EngineStop = Sound("ATV_engine_stop")

function ENT:Initialize()
	self:SetModel(self.Model)
	self:SetPos(self:GetPos() + self.CreationOffset)

	self:PhysicsInit(SOLID_VPHYSICS)
	self:SetSolid(SOLID_VPHYSICS)

	self.DesiredCannonAngle = 0

	self:CreateChildren()

	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		phys:SetMaterial("metal")
		phys:SetMass(500)
		phys:EnableMotion(true)
		phys:Wake()
	end

	self:StartMotionController()

	self:SetVHealth(self.MaxHealth)

	self.NextShoot = 0
	self.NextFastIdle = 0

	self.Destroy = CurTime() + 30

	self.IdleSound = CreateSound(self, EngineIdle)
	self.IdleSlowSound = CreateSound(self, EngineIdleSlow)
	self.StartSound = CreateSound(self, EngineStart)
	self.ForwardSound = CreateSound(self, EngineForward)

	self.LastAttacked = 0
	self.LastDriver = NULL
	self.LastGunner = NULL

	self.Attacker = NULL
	self.Inflictor = NULL

	--self.DesiredYaw = 0
end

function ENT:OnRemove()
	self.IdleSound:Stop()
	self.IdleSlowSound:Stop()
	self.StartSound:Stop()
	self.ForwardSound:Stop()
end

function ENT:UpdateCrew()
	if self:IsValid() then
		umsg.Start("RecARD")
			umsg.Entity(self)
			umsg.Entity(self.LastDriver)
			umsg.Entity(self.LastGunner)
		umsg.End()

		local lastdriverindex = 0
		if self.LastDriver:IsValid() then lastdriverindex = self.LastDriver:EntIndex() end
		local lastgunnerindex = 0
		if self.LastGunner:IsValid() then lastgunnerindex = self.LastGunner:EntIndex() end
		gmod.BroadcastLua("RecARD("..self:EntIndex()..","..lastdriverindex..","..lastgunnerindex..")")
	end
end

function ENT:PilotEnter(pl)
	self:SetTeam(pl:Team())
	self.LastDriver = pl
	self.Destroy = 99999999
	self.StartSound:PlayEx(1, 120)
	self.PlayingIdle = true
	self:SetNetworkedBool("on", true)
	local gunner = self.GunnerSeat:GetDriver()
	if gunner:IsValid() and gunner:Team() ~= self:Team() then
		gunner:ExitVehicle()
	end

	self.FrontRightWheel:SetOwner(pl)
	self.BackRightWheel:SetOwner(pl)
	self.FrontLeftWheel:SetOwner(pl)
	self.BackLeftWheel:SetOwner(pl)
end

function ENT:PilotExit(pl)
	self.LastDriver = NULL

	self.FrontRightWheel:SetOwner(NULL)
	self.BackRightWheel:SetOwner(NULL)
	self.FrontLeftWheel:SetOwner(NULL)
	self.BackLeftWheel:SetOwner(NULL)
end

function ENT:GunnerEnter(pl)
	self.LastGunner = pl
end

function ENT:Exit(pl, veh, role)
	if veh == self.PilotSeat then
		self:PilotExit(pl)
	elseif veh == self.GunnerSeat then
		self:GunnerExit(pl)
	end
	timer.Simple(0.25, self.UpdateCrew, self)
end

function ENT:GunnerExit(pl)
	self.LastGunner = NULL
end

function ENT:Enter(pl, veh, role)
	if veh == self.PilotSeat then
		self:PilotEnter(pl)
	elseif veh == self.GunnerSeat then
		self:GunnerEnter(pl)
	end
	timer.Simple(0.25, self.UpdateCrew, self)
end

function ENT:CreateChildren()
	local vPos = self:GetPos()

	local ent = ents.Create("prop_vehicle_prisoner_pod")
	if ent:IsValid() then
		ent:SetKeyValue("model", "models/props_phx/carseat2.mdl")
		ent:SetKeyValue("solid", "6")
		ent:SetKeyValue("vehiclescript", "scripts/vehicles/prisoner_pod.txt")
		ent:SetPos(vPos + Vector(15, 13, -3))
		ent:SetAngles(Angle(0, 270, 0))
		ent:Spawn()
		ent:SetVehicle(self)
		constraint.Weld(self, ent, 0, 0, 0, 1)
		self:DeleteOnRemove(ent)
		self.PilotSeat = ent
		ent:SetNetworkedEntity("vehicle", self)
		self:SetNetworkedEntity("driverseat", ent)
		ent:SetMaterial("models/debug/debugwhite")
		ent.Protects = true
	end

	local ent = ents.Create("prop_vehicle_prisoner_pod")
	if ent:IsValid() then
		ent:SetKeyValue("model", "models/props_phx/carseat2.mdl")
		ent:SetKeyValue("solid", "6")
		ent:SetKeyValue("vehiclescript", "scripts/vehicles/prisoner_pod.txt")
		ent:SetPos(vPos + Vector(-32, 0, 44))
		ent:SetAngles(Angle(0, 270, 0))
		ent:Spawn()
		ent.:SetVehicle(self)
		ent:GetPhysicsObject():SetMass(20)
		constraint.Axis(ent, self, 0, 0, Vector(0, 0, -16), Vector(-32, 0, 21), 0, 0, 0, 1)
		self:DeleteOnRemove(ent)
		self.GunnerSeat = ent
		ent:SetNetworkedEntity("vehicle", self)
		self:SetNetworkedEntity("gunnerseat", ent)
		ent:SetMaterial("models/debug/debugwhite")
		ent.Protects = true
		local cannon = ents.Create("prop_dynamic_override")
		if cannon:IsValid() then
			cannon:SetModel("models/props_borealis/bluebarrel001.mdl")
			cannon:SetPos(ent:GetPos() + Vector(16, -24, 16))
			cannon:SetAngles(Angle(90, 0, 0))
			cannon:Spawn()
			cannon.:SetVehicle(self)
			--cannon:GetPhysicsObject():SetMass(10)
			self:DeleteOnRemove(cannon)
			self.Cannon = cannon
			--constraint.Weld(ent, cannon, 0, 0, 0, 1)
			cannon:SetParent(ent)
			self:SetNetworkedEntity("cannon", cannon)
		end
	end

	local engine = ents.Create("vehiclepart")
	if engine:IsValid() then
		engine:SetModel("models/props_c17/trappropeller_engine.mdl")
		engine:SetPos(vPos + Vector(54, 0, 12))
		engine:SetAngles(Angle(90, 180, 0))
		engine:Spawn()
		engine.:SetVehicle(self)
		engine:GetPhysicsObject():SetMass(10)
		constraint.Weld(self, engine, 0, 0, 0, 1)
		self:DeleteOnRemove(engine)
		self.Engine = engine
	end

	local wheel = ents.Create("roverwheel")
	if wheel:IsValid() then
		wheel:SetPos(vPos + Vector(50, 38, -16))
		wheel:Spawn()
		wheel:SetFriction(50)
		wheel.:SetVehicle(self)
		wheel:GetPhysicsObject():SetMass(170)
		constraint.Axis(wheel, self, 0, 0, Vector(0, 12, 0), Vector(50, 38, -16), 0, 0, 3, 1)
		self:DeleteOnRemove(wheel)
		self.FrontLeftWheel = wheel
	end

	local wheel = ents.Create("roverwheel")
	if wheel:IsValid() then
		wheel:SetPos(vPos + Vector(50, -38, -16))
		wheel:Spawn()
		wheel:SetFriction(50)
		wheel.:SetVehicle(self)
		wheel:GetPhysicsObject():SetMass(170)
		constraint.Axis(wheel, self, 0, 0, Vector(0, -12, 0), Vector(50, -38, -16), 0, 0, 3, 1)
		self:DeleteOnRemove(wheel)
		self.FrontRightWheel = wheel
	end

	local wheel = ents.Create("roverwheel")
	if wheel:IsValid() then
		wheel:SetPos(vPos + Vector(-50, 38, -16))
		wheel:Spawn()
		wheel:SetFriction(50)
		wheel.:SetVehicle(self)
		wheel:GetPhysicsObject():SetMass(240)
		constraint.Axis(wheel, self, 0, 0, Vector(0, 12, 0), Vector(-50, 38, -16), 0, 0, 3, 1)
		self:DeleteOnRemove(wheel)
		self.BackLeftWheel = wheel
	end

	local wheel = ents.Create("roverwheel")
	if wheel:IsValid() then
		wheel:SetPos(vPos + Vector(-50, -36, -16))
		wheel:Spawn()
		wheel:SetFriction(50)
		wheel.:SetVehicle(self)
		wheel:GetPhysicsObject():SetMass(240)
		constraint.Axis(wheel, self, 0, 0, Vector(0, -12, 0), Vector(-50, -38, -16), 0, 0, 3, 1)
		self:DeleteOnRemove(wheel)
		self.BackRightWheel = wheel
	end
end

function ENT:PhysicsSimulate(phys, frametime)
	phys:Wake()

	local isMDF = self.MDF and CurTime() <= self.MDF

	local gunner = self.GunnerSeat:GetDriver()
	if gunner:IsValid() then
		if not isMDF then
			if gunner:KeyDown(IN_MOVELEFT) then
				self.DesiredCannonAngle = math.NormalizeAngle(self.DesiredCannonAngle + frametime * 120)
				--newangles:RotateAroundAxis(newangles:Up(), frametime * 120)
			elseif gunner:KeyDown(IN_MOVERIGHT) then
				self.DesiredCannonAngle = math.NormalizeAngle(self.DesiredCannonAngle - frametime * 120)
				--newangles:RotateAroundAxis(newangles:Up(), frametime * -120)
			end
		end

		local newangles = self.GunnerSeat:GetAngles()
		local diffangles = self.GunnerSeat:WorldToLocalAngles(Angle(0, self.DesiredCannonAngle, 0))
		newangles:RotateAroundAxis(newangles:Up(), diffangles.yaw * frametime * 120)
		self.GunnerSeat:SetAngles(newangles)
	end

	local vel = self:GetVelocity()
	local gphys = self.GunnerSeat:GetPhysicsObject()
	gphys:SetVelocityInstantaneous(vel)

	local driver = self.PilotSeat:GetDriver()
	if not driver:IsValid() or isMDF then
		--[[if self.LastDriver:IsValid() then
			self:PilotExit(self.LastDriver)
			self.LastDriver = NULL
		end]]

		if self.PlayingIdle then
			self.IdleSound:Stop()
			self.IdleSlowSound:Stop()
			self.StartSound:Stop()
			self.PlayingIdle = nil
		end

		if self.PlayingForward then
			self.ForwardSound:Stop()
			self.PlayingForward = nil
		end

		return SIM_NOTHING
	end

	self.FrontLeftWheel:SetPhysicsAttacker(driver)
	self.FrontRightWheel:SetPhysicsAttacker(driver)
	self.BackLeftWheel:SetPhysicsAttacker(driver)
	self.BackRightWheel:SetPhysicsAttacker(driver)

	if driver:KeyDown(IN_FORWARD) then
		local downoffset = self:GetUp() * -27
		local backleftwheelpos = self.BackLeftWheel:GetPos()
		local backrightwheelpos = self.BackRightWheel:GetPos()

		if util.TraceLine({start=backleftwheelpos, endpos=backleftwheelpos + downoffset, filter=self.BackLeftWheel}).Hit or util.TraceLine({start=backrightwheelpos, endpos=backrightwheelpos + downoffset, filter=self.BackRightWheel}).Hit then
			local targetvel = 64000 * frametime * self:GetForward()
			local veldist = phys:GetVelocity():Distance(targetvel)
			if 200 < veldist then
				phys:ApplyForceCenter(targetvel * 12)
			end
		end

		if self.PlayingIdle then
			self.IdleSound:Stop()
			self.IdleSlowSound:Stop()
			self.StartSound:Stop()
			self.PlayingIdle = nil
		end

		if not self.PlayingForward then
			self.ForwardSound:PlayEx(1, 120)
			self.PlayingForward = true
			self.NextFastIdle = CurTime() + 2
		end
	elseif driver:KeyDown(IN_BACK) then
		local targetvel = -64000 * frametime * self:GetForward()
		local veldist = phys:GetVelocity():Distance(targetvel)

		if 200 < veldist then
			phys:ApplyForceCenter(targetvel * 7)
		end

		if self.PlayingIdle then
			self.IdleSound:Stop()
			self.IdleSlowSound:Stop()
			self.StartSound:Stop()
			self.PlayingIdle = nil
		end

		if not self.PlayingForward then
			self.ForwardSound:PlayEx(1, 120)
			self.PlayingForward = true
			self.NextFastIdle = CurTime() + 2
		end
	elseif not self.PlayingIdle then
		self.PlayingIdle = true

		if CurTime() > self.NextFastIdle then
			self.IdleSound:PlayEx(1, 120)
		else
			self.IdleSlowSound:PlayEx(1, 120)
		end

		if self.PlayingForward then
			self.ForwardSound:Stop()
			self.PlayingForward = nil
		end
	end

	if driver:KeyDown(IN_MOVELEFT) then
		local downoffset = self:GetUp() * -30
		local backleftwheelpos = self.FrontLeftWheel:GetPos()
		local backrightwheelpos = self.FrontRightWheel:GetPos()

		if util.TraceLine({start=backleftwheelpos, endpos=backleftwheelpos + downoffset, filter=self.FrontLeftWheel}).Hit or util.TraceLine({start=backrightwheelpos, endpos=backrightwheelpos + downoffset, filter=self.FrontRightWheel}).Hit then
			if phys:GetAngleVelocity().z < 230 then
				phys:AddAngleVelocity(Vector(0, 0, 1600 * frametime))
			end
		end
	elseif driver:KeyDown(IN_MOVERIGHT) then
		local downoffset = self:GetUp() * -30
		local backleftwheelpos = self.FrontLeftWheel:GetPos()
		local backrightwheelpos = self.FrontRightWheel:GetPos()

		if util.TraceLine({start=backleftwheelpos, endpos=backleftwheelpos + downoffset, filter=self.BackLeftWheel}).Hit or util.TraceLine({start=backrightwheelpos, endpos=backrightwheelpos + downoffset, filter=self.FrontRightWheel}).Hit then
			if phys:GetAngleVelocity().z > -230 then
				phys:AddAngleVelocity(Vector(0, 0, -1600 * frametime))
			end
		end
	end

	return SIM_NOTHING
end

function ENT:Think()
	if self.DoDestroy or 0 < self.GunnerSeat:WaterLevel() then
		self:Destroyed(self.Attacker or NULL, self.Inflictor or NULL)
		self:Remove()
		return
	end

	local gunner = self.GunnerSeat:GetDriver()
	if gunner:IsValid() and not (self.MDF and CurTime() <= self.MDF) then
		if gunner:KeyDown(IN_ATTACK) and self.NextShoot < CurTime() then
			self.NextShoot = CurTime() + 1.25

			local dir = self.Cannon:GetUp()
			dir.z = math.Clamp(gunner:GetAimVector().z, dir.z - 0.6, dir.z + 0.6)
			dir = dir:Normalize()

			local ent = ents.Create("projectile_assaultrovercannon")
			if ent:IsValid() then
				ent:SetPos(self.Cannon:GetPos() + dir * 40)
				ent:SetAngles(dir:Angle())
				ent:SetOwner(gunner)
				ent:SetSkin(self:Team())
				ent:Spawn()
				ent:SetTeamID(gunner:Team())
				self.Cannon:EmitSound("vehicles/Airboat/pontoon_impact_hard1.wav", 80, math.Rand(95, 105))
				local phys = ent:GetPhysicsObject()
				if phys:IsValid() then
					phys:SetVelocityInstantaneous(dir * 1400 + self:GetVelocity() * 0.5)
				end
			end
		end
	end

	if 2 < self:WaterLevel() then
		local world = GetWorldEntity()
		self.Attacker = world
		self.Inflictor = world
		self.DoDestroy = true
	elseif CurTime() > self.Destroy then
		for _, ent in pairs(ents.FindInSphere(self:GetPos(), 512)) do
			if ent.SendLua and ent:Alive() then
				self.Destroy = CurTime() + 15
				return
			end
		end
		self:Remove()
	elseif self.Destroy == 99999999 then
		self.Destroy = CurTime() + 45
		self:EmitSound(EngineStop)
		if self.PlayingForward then
			self.ForwardSound:Stop()
			self.PlayingForward = nil
		end
		if self.PlayingIdle then
			self.IdleSound:Stop()
			self.IdleSlowSound:Stop()
			self.StartSound:Stop()
			self.PlayingIdle = nil
		end
		self:SetNetworkedBool("on", false)
	end
end

function ENT:TakeDamage(amount, attacker, inflictor)
	self:TakeSpecialDamage(amount, DMGTYPE_GENERIC, attacker, inflictor)
end

local RandomProps = {"models/props_vehicles/carparts_axel01a.mdl",
"models/props_vehicles/carparts_door01a.mdl",
"models/props_vehicles/carparts_door01a.mdl",
"models/props_vehicles/carparts_muffler01a.mdl",
"models/props_c17/pulleywheels_large01.mdl"
}

function ENT:Destroyed(attacker, inflictor)
	if self.AlreadyDestroyed then return end
	self.AlreadyDestroyed = true
	self.PilotSeat.Protects = nil
	self.GunnerSeat.Protects = nil

	attacker = attacker or NULL

	if not attacker:IsValid() and self.Attacker:IsValid() and CurTime() < self.LastAttacked + 10 then
		attacker = self.Attacker
	end

	if not attacker:IsValid() and self.LastDriver:IsValid() then
		attacker = self.LastDriver
	end

	local driver = self.PilotSeat:GetDriver()
	local driverisvalid = driver:IsValid()

	if driverisvalid then
		driver:GodDisable()
		driver.Shield = nil
		driver:RemoveStatus("forcefield", false, true)
		driver:TakeSpecialDamage(500, DMGTYPE_FIRE, attacker, inflictor)
	end

	local gunner = self.GunnerSeat:GetDriver()
	if gunner:IsValid() then
		gunner:GodDisable()
		gunner.Shield = nil
		gunner:RemoveStatus("forcefield", false, true)
		gunner:TakeSpecialDamage(500, DMGTYPE_FIRE, attacker, inflictor)
	end

	ExplosiveDamage(attacker, self, self:GetPos(), 425, 425, 1, 0.4, 8)

	local vel = self:GetVelocity()
	local vel65 = vel * 0.65

	local pos = self:GetPos()
	local effectdata = EffectData()
		effectdata:SetOrigin(pos)
	util.Effect("FireBombExplosion", effectdata)

	for _, model in pairs(RandomProps) do
		local ent = ents.Create("prop_physics_multiplayer")
		if ent:IsValid() then
			ent:SetPos(pos + Vector(math.Rand(-64, 64), math.Rand(-64, 64), math.Rand(0, 48)))
			ent:SetAngles(VectorRand():Angle())
			ent:SetModel(model)
			ent:Spawn()
			local phys = ent:GetPhysicsObject()
			if phys:IsValid() then
				phys:ApplyForceCenter(vel65 + Vector(0, 0, 1050) + VectorRand() * 900 * phys:GetMass())
				phys:AddAngleVelocity(VectorRand() * 300)
			end
			if driverisvalid then
				ent:SetPhysicsAttacker(driver)
			end
			ent:Fire("kill", "", 14)
			if math.random(1, 3) == 1 then
				local fire = ents.Create("env_fire_trail")
				if fire:IsValid() then
					fire:SetPos(ent:GetPos())
					fire:Spawn()
					fire:SetParent(ent)
				end
			end
		end
	end

	local ent2 = ents.Create("prop_physics_multiplayer")
	if ent2:IsValid() then
		ent2:SetPos(self.FrontLeftWheel:GetPos())
		ent2:SetAngles(self.FrontLeftWheel:GetAngles())
		ent2:SetModel(self.FrontLeftWheel:GetModel())
		ent2:Spawn()
		local phys = ent2:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(self.FrontLeftWheel:GetVelocity() + VectorRand() * 2250 + Vector(0, 0, 800))
			phys:ApplyForceCenter(vel65 + Vector(0, 0, 1050) + VectorRand() * 900 * phys:GetMass())
			phys:AddAngleVelocity(VectorRand() * 200)
		end
		if driverisvalid then
			ent2:SetPhysicsAttacker(driver)
		end
		ent2:Fire("kill", "", 14)
		local fire = ents.Create("env_fire_trail")
		if fire:IsValid() then
			fire:SetPos(ent2:GetPos())
			fire:Spawn()
			fire:SetParent(ent2)
		end
	end

	local ent2 = ents.Create("prop_physics_multiplayer")
	if ent2:IsValid() then
		ent2:SetPos(self.Engine:GetPos())
		ent2:SetAngles(self.Engine:GetAngles())
		ent2:SetModel(self.Engine:GetModel())
		ent2:Spawn()
		local phys = ent2:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(self.Engine:GetVelocity() + VectorRand() * 850 + Vector(0, 0, 2250))
			phys:ApplyForceCenter(vel65 + Vector(0, 0, 1050) + VectorRand() * 900 * phys:GetMass())
			phys:AddAngleVelocity(VectorRand() * 300)
		end
		if driverisvalid then
			ent2:SetPhysicsAttacker(driver)
		end
		ent2:Fire("kill", "", 14)
		local fire = ents.Create("env_fire_trail")
		if fire:IsValid() then
			fire:SetPos(ent2:GetPos())
			fire:Spawn()
			fire:SetParent(ent2)
		end
	end

	local ent2 = ents.Create("prop_physics_multiplayer")
	if ent2:IsValid() then
		ent2:SetPos(self.FrontRightWheel:GetPos())
		ent2:SetAngles(self.FrontRightWheel:GetAngles())
		ent2:SetModel(self.FrontRightWheel:GetModel())
		ent2:Spawn()
		local phys = ent2:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(self.FrontRightWheel:GetVelocity() + VectorRand() * 1250 + Vector(0, 0, 1800))
			phys:ApplyForceCenter(vel65 + Vector(0, 0, 1050) + VectorRand() * 900 * phys:GetMass())
			phys:AddAngleVelocity(VectorRand() * 300)
		end
		if driverisvalid then
			ent2:SetPhysicsAttacker(driver)
		end
		ent2:Fire("kill", "", 14)
		local fire = ents.Create("env_fire_trail")
		if fire:IsValid() then
			fire:SetPos(ent2:GetPos())
			fire:Spawn()
			fire:SetParent(ent2)
		end
	end

	local ent2 = ents.Create("prop_physics_multiplayer")
	if ent2:IsValid() then
		ent2:SetPos(self.BackLeftWheel:GetPos())
		ent2:SetAngles(self.BackLeftWheel:GetAngles())
		ent2:SetModel(self.BackLeftWheel:GetModel())
		ent2:Spawn()
		local phys = ent2:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(self.BackLeftWheel:GetVelocity() + VectorRand() * 1250 + Vector(0, 0, 1800))
			phys:ApplyForceCenter(vel65 + Vector(0, 0, 1050) + VectorRand() * 900 * phys:GetMass())
			phys:AddAngleVelocity(VectorRand() * 300)
		end
		if driverisvalid then
			ent2:SetPhysicsAttacker(driver)
		end
		ent2:Fire("kill", "", 14)
		local fire = ents.Create("env_fire_trail")
		if fire:IsValid() then
			fire:SetPos(ent2:GetPos())
			fire:Spawn()
			fire:SetParent(ent2)
		end
	end

	local ent2 = ents.Create("prop_physics_multiplayer")
	if ent2:IsValid() then
		ent2:SetPos(self.BackRightWheel:GetPos())
		ent2:SetAngles(self.BackRightWheel:GetAngles())
		ent2:SetModel(self.BackRightWheel:GetModel())
		ent2:Spawn()
		local phys = ent2:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(self.BackRightWheel:GetVelocity() + VectorRand() * 1250 + Vector(0, 0, 1800))
			phys:ApplyForceCenter(vel65 + Vector(0, 0, 1050) + VectorRand() * 900 * phys:GetMass())
			phys:AddAngleVelocity(VectorRand() * 300)
		end
		if driverisvalid then
			ent2:SetPhysicsAttacker(driver)
		end
		ent2:Fire("kill", "", 14)
		local fire = ents.Create("env_fire_trail")
		if fire:IsValid() then
			fire:SetPos(ent2:GetPos())
			fire:Spawn()
			fire:SetParent(ent2)
		end
	end
end

function ENT:ProcessSpecialDamage(amount, dmgtype, attacker, inflictor)
	if dmgtype == DMGTYPE_POISON then return 0, dmgtype, attacker, inflictor end

	if dmgtype ~= DMGTYPE_GENERIC and dmgtype ~= DMGTYPE_PHYSICAL then
		self.IgnoreDamageTime = CurTime()
	end

	if dmgtype == DMGTYPE_FIRE or dmgtype == DMGTYPE_BULLET then
		amount = amount * 2
	end

	return amount, dmgtype, attacker, inflictor
end

function ENT:OverrideTakeDamage(amount, attacker, inflictor)
	local myteam = self:GetTeamID()
	if attacker:IsValid() and attacker:GetTeamID() == myteam then return 0, attacker, inflictor end

	if attacker:IsValid() then
		self.LastAttacked = CurTime()
		self.Attacker = attacker
	end
	self.Inflictor = inflictor

	self:SetVHealth(self:VHealth() - amount)

	local newhealth = self:VHealth()

	local driver = self.PilotSeat:GetDriver()
	if driver:IsValid() then
		driver.LastAttacker = attacker
		driver.LastAttacked = CurTime()
	end

	if newhealth <= 0 then
		if attacker:IsValid() then
			self.Attacker = attacker
		end
		self.Inflictor = inflictor
		self.DoDestroy = true
	elseif newhealth < self.MaxHealth * 0.3 and not self.Ignited then
		local effectdata = EffectData()
		effectdata:SetMagnitude(1)
		effectdata:SetScale(1)
		effectdata:SetRadius(1)
		for i=1, math.random(1, 2) do
			local pos = self:GetPos() + VectorRand() * 32
			local ent = ents.Create("env_fire_trail")
			if ent:IsValid() then
				ent:SetPos(pos)
				ent:Spawn()
				ent:SetParent(self)
				effectdata:SetOrigin(pos)
				util.Effect("Explosion", effectdata)
			end
		end
		self.Ignited = true
	end
end
