AddCSLuaFile("cl_blitz.lua")

function GM:BlitzInitialize()
	self.Name = self.Name.." (Blitz)"
	self.GameType = "Blitz"
	self.FlagEntity = "flag_blitz"
	gmod.BroadcastLua("GAMEMODE:BlitzInitialize()")
	hook.Add("PlayerReady", "GMSend2", function(pl)
		pl:SendLua("GAMEMODE:BlitzInitialize()")
	end)

	for _, ent in pairs(ents.FindByClass("logic_gametypeinitialize")) do
		if ent.GameType == self.GameType then
			ent:FireOutput("OnInitialize", ent, ent)
		end
	end

	local ent = ents.Create(self.FlagEntity)
	if ent:IsValid() then
		ent:SetPos(self.BallDrop)
		ent:Spawn()
		self.Flag = ent
		self.Ball = ent
	end

	for i, teamid in pairs(TEAMS_PLAYING) do
		local tab = team.TeamInfo[teamid]
		if tab.FlagPoint then
			local ent = ents.Create("prop_blitzgoal")
			if ent:IsValid() then
				local entpos = tab.FlagPoint
				ent:SetPos(entpos)
				local pos = Vector(self.BallDrop.x, self.BallDrop.y, self.BallDrop.z)
				pos.z = entpos.z
				ent:SetAngles((pos - entpos):Angle())
				ent:SetTeamID(teamid)
				local col = team.GetColor(teamid)
				ent:SetColor(col.r, col.g, col.b, 255)
				ent:Spawn()
				tab.Flag = ent
				ent:DropToFloor()
			end
		end
	end

	function self:BallReset()
		umsg.Start("BallReset")
		umsg.End()
	end

	function self:BallTaken(pl)
		umsg.Start("BallTaken")
			umsg.Entity(pl)
			umsg.Short(pl:GetTeamID())
		umsg.End()
	end

	function self:BallDropped(pl)
		if pl and pl:IsValid() then
			umsg.Start("BallDropped")
				umsg.Entity(pl)
				umsg.Short(pl:GetTeamID())
			umsg.End()
		end
	end

	function self:BallScored(pl, teamid, ball, goal)
		if ENDGAME or self.Intermission then return end

		team.AddScore(teamid, 1)

		pl:AddFrags(BLITZ_SCORE_FRAGS)
		pl:AddMoney(BLITZ_SCORE_MONEY)
		-- TODO: Enable this
		--pl.BlitzScores = pl.BlitsScores + 1

		umsg.Start("BallScored")
			umsg.Entity(pl)
			umsg.Short(teamid)
			umsg.Entity(ball)
			umsg.Entity(goal)
		umsg.End()

		local pos, ang = goal:GetPos() + Vector(0, 0, 32), Angle(0, 0, 0)
		ParticleEffect("citadel_shockwave", pos, ang)
		ParticleEffect("aurora_shockwave", pos, ang)

		if CurTime() <= NEXT_FIGHTMODE + 30 and not pl:HasAward("Stealing_First") then
			pl:GiveAward("Stealing_First")
		end

		pl.CapturesThisRound = (pl.CapturesThisRound or 0) + 1

		if BLITZ_SCORE_LIMIT <= team.GetScore(teamid) then
			self:EndGame(teamid, {})
			if pl.CapturesThisRound == BLITZ_SCORE_LIMIT and not pl:HasAward("Greedy") then
				pl:GiveAward("Greedy")
			end
		else
			gamemode.Call("StartIntermission", pl)
			timer.Simple(BLITZ_INTERMISSION_TIME - 0.125, gamemode.Call, "CleanUpForNewRound")
			timer.Simple(BLITZ_INTERMISSION_TIME, gamemode.Call, "EndIntermission")
		end
	end

	function self:HandleOvertime()
		local topscore = 0
		local topteam = 0
		for _, teamid in pairs(TEAMS_PLAYING) do
			if topscore < team.GetScore(teamid) then
				topscore = team.GetScore(teamid)
				topteam = teamid
			end
		end
		local numwin = 0
		for _, teamid in pairs(TEAMS_PLAYING) do
			if topscore == team.GetScore(teamid) then
				numwin = numwin + 1
			end
		end

		if numwin == 1 then
			self:EndGame(topteam, {})
		else
			self:EndGame(0, {})
		end
	end

	function SortTeams(a, b)
		local aamount = team.NumPlayers(a)
		local bamount = team.NumPlayers(b)

		if aamount < bamount * 1.2 and bamount * 0.8 < aamount then
			if BUILD_LENGTH < CurTime() then
				local ascore = team.GetScore(a)
				local bscore = team.GetScore(b)
				if ascore == bscore then
					return team.TotalFrags(a) < team.TotalFrags(b)
				else
					return ascore < bscore
				end
			else
				return team.TotalWeight(a) < team.TotalWeight(b)
			end
		end

		return aamount < bamount
	end
end
