spells = {}

function GM:PlayerCanCast(pl, spell)
	return not pl:InVehicle() and not pl.Stunned
end

local function ForwardSpell(func, pl, spellid, precast, ignorerequirements)
	local spelltab = Spells[spellid]
	if pl:IsValid() then
		if not pl:Alive() then
			pl.Casting = nil
			return
		end

		local wep = pl:GetActiveWeapon()
		if not wep.CanCast then
			pl:LMR(60)
			return
		end

		local mana
		local stamina

		if not ignorerequirements then
			mana = pl:GetMana()
			stamina = pl:GetStamina()

			if 0 < spelltab.Mana then
				if pl.ManaStunned then
					umsg.Start("mastu", pl)
					umsg.End()
					return
				end

				if mana < spelltab.Mana then
					umsg.Start("insma", pl)
					umsg.End()
					return
				end
			end
			if 0 < spelltab.Stamina and stamina < spelltab.Stamina then
				umsg.Start("insst", pl)
				umsg.End()
				return
			end
		end

		if precast then
			if not ignorerequirements then
				if 0 < spelltab.Mana then
					pl:SetMana(mana - spelltab.Mana, true)
				end
				if 0 < spelltab.Stamina then
					pl:SetStamina(stamina - spelltab.Stamina, true)
				end
			end

			if wep.PreCastedCallback then
				wep:PreCastedCallback(pl, spellid, spelltab)
			end
		elseif func then
			if func(pl) then
				if wep.FailCastedCallback then
					wep:FailCastedCallback(pl, spellid, spelltab)
				end
			else
				if pl.Sprinting then
					pl:RemoveStatus("sprint*", false, true)
				end
				if pl.DashDodging then
					pl:RemoveStatus("dashdodge*", false, true)
				end

				if not ignorerequirements then
					if 0 < spelltab.Mana then
						pl:SetMana(mana - spelltab.Mana, true)
					end
					if 0 < spelltab.Stamina then
						pl:SetStamina(stamina - spelltab.Stamina, true)
					end
				end

				if wep.CastedCallback then
					wep:CastedCallback(pl, spellid, spelltab)
				end
			end
		end
	end
end

local function Cast(sender, command, arguments)
	if arguments[1] == nil or not sender:Alive() or sender:IsFrozen() or CurTime() < sender.NextSpell then return end

	local spellname
	local spellid

	local wep = sender:GetActiveWeapon()
	local precastedid = wep.PreCastedSpell
	if precastedid then
		spellid = precastedid
		spellname = Spells[spellid].Name
	else
		spellname = table.concat(arguments, " ")
		spellid = NameToSpellLower[string.lower(spellname)]
	end

	if GAMEMODE.DisabledSpells[spellid] then
		sender:LMR(54)
		return
	end

	if not spellid then
		sender:LMR(23)
		return
	end

	if not gamemode.Call("PlayerCanCast", sender, spellid) then return end

	if not wep.CanCast then
		sender:LMR(60)
		return
	end

	local spelltab = Spells[spellid]

	if precastedid then
		ForwardSpell(SpellFunctions[spellid], sender, spellid, nil, true)

		return
	end

	local aspects = sender.Aspects
	for i=1, NUMASPECTS do
		local req = spelltab[i]
		if req and aspects[i] < req then
			return sender:PrintMessage(HUD_PRINTCENTER, "You need at least ".. req * 100 .." "..ASPECTS[i].Name.." to cast "..spelltab.Name)
		end
	end

	local mana = sender:GetMana()
	local stamina = sender:GetStamina()

	if 0 < spelltab.Mana then
		if sender.ManaStunned then
			umsg.Start("mastu", sender)
			umsg.End()
			return
		end

		if mana < spelltab.Mana then
			umsg.Start("insma", sender)
			umsg.End()
			return
		end
	end

	if 0 < spelltab.Stamina and stamina < spelltab.Stamina then
		umsg.Start("insst", sender)
		umsg.End()
		return
	end

	if sender.Sprinting then
		sender:RemoveStatus("sprint*", false, true)
	end
	if sender.DashDodging then
		sender:RemoveStatus("dashdodge*", false, true)
	end

	if spelltab.Words then
		local mul = sender.CastingRateMultiplier
		local delay = spelltab.Delay * mul
		timer.Create("DoSpellFor"..sender:UniqueID(), delay, 1, ForwardSpell, SpellFunctions[spellid], sender, spellid, wep.PreCast)
		sender.CantDodgeTime = math.max(sender.CantDodgeTime or 0, CurTime() + delay + 0.35)
		sender.CantSprintTime = math.max(sender.CantSprintTime or 0, CurTime() + delay + 0.35)
		sender.Casting = CurTime() + delay + 0.5
		umsg.Start("SI")
			umsg.Entity(sender)
			umsg.Float(mul)
			umsg.Short(spellid)
		umsg.End()
		sender.NextSpell = CurTime() + delay
	elseif wep.PreCast then
		ForwardSpell(SpellFunctions[spellid], sender, spellid, true)
	elseif not SpellFunctions[spellid](sender) then
		if 0 < spelltab.Mana then
			sender:SetMana(mana - spelltab.Mana, true)
		end
		if 0 < spelltab.Stamina then
			sender:SetStamina(stamina - spelltab.Stamina, true)
		end
	end
end
concommand.Add("cast", Cast)
concommand.Add("spell", Cast)

function EmulatedCast(pl, command, arguments)
	Cast(pl, command, arguments)
end

function spells.SmartSound(pl, sound)
	if pl:IsValid() and pl:Alive() then
		pl:EmitSound(sound)
	end
end

function spells.FlameSpiral(pl)
	local ent = ents.Create("projectile_flamespiral")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		ent.ProjectileHeading = pl:GetAimVector()
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:ApplyForceCenter(ent.ProjectileHeading * 250)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.FireBolt(pl)
	local ent = ents.Create("projectile_firebolt")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:ApplyForceCenter(pl:GetAimVector() * 700)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.FlameStrike(pl)
	local ent = ents.Create("projectile_flamestrike")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:ApplyForceCenter(pl:GetAimVector() * 700)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.Flare(pl)
	local ent = ents.Create("projectile_flare")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetColor(pl:GetColor())
		ent:SetPos(pl:GetShootPos())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 600)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.ElectronBall(pl)
	local ent = ents.Create("projectile_electronball")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 600)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.Flash(pl)
	local ent = ents.Create("projectile_flash")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 800)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.Sparkler(pl)
	local ent = ents.Create("projectile_sparkler")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetPos(pl:GetShootPos())
		local teamid = pl:Team()
		ent:SetTeamID(teamid)
		local col = team.GetColor(teamid)
		ent:SetColor(col.r, col.g, col.b, 255)
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 900)
		end
	end

	pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
end

function spells.MagicArrow(pl)
	local ent = ents.Create("projectile_magicarrow")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		local r,g,b = pl:GetColor()
		ent:SetColor(r, g, b, 255)
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 1350)
		end
	end
end

function spells.StarBurst(pl)
	local ent = ents.Create("projectile_starburst")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		ent.Target = pl:GetEyeTrace().HitPos
		ent:Spawn()
		ent:SetColor(pl:GetColor())
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:Wake()
			phys:SetVelocityInstantaneous(Vector(0,0,900))
		end
	end
end

function spells.Scatterfrost(pl)
	local ent = ents.Create("projectile_scatterfrost")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		local aimvec = pl:GetAimVector()
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:Wake()
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 550)
		end

		pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
	end
end

function spells.FireBomb(pl)
	local ent = ents.Create("projectile_firebomb")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		ent:SetPos(pl:GetShootPos())
		local r,g,b = pl:GetColor()
		ent:SetColor(r, g, b, 255)
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			local aimvec = pl:GetAimVector()
			aimvec.z = math.max(aimvec.z, 0.35)
			aimvec = aimvec:Normalize()
			phys:SetVelocityInstantaneous(aimvec * 850)
		end
		util.SpriteTrail(ent, 0, color_white, false, 48, 32, 1, 0.025, "Effects/fire_cloud2.vmt")

		pl:RestartGesture(ACT_SIGNAL_FORWARD)
	end
end

function BurnTimer(fire, tn, damage, radius)
	if not fire:IsValid() then timer.Destroy(tn) return end

	local owner = fire:GetOwner()
	if not owner:IsValid() then
		owner = fire
	end
	for _, ent in pairs(ents.FindInSphere(fire:GetPos(), radius)) do
		if not ent.CoreHealth and not ent.PHealth and ent:GetClass() ~= "ffdoor" then
			ent:TakeSpecialDamage(damage, DMGTYPE_FIRE, owner, DUMMY_BURN)
		end
	end
end

function spells.Burn(pl)
	local filt = ents.FindByClass("projectile_*")
	table.insert(filt, pl)
	local tr = util.TraceLine({start = pl:GetShootPos(), endpos = pl:GetShootPos() + pl:GetAimVector() * 1000, filter=filt})
	if not tr.Hit then
		tr = util.TraceLine({start = tr.HitPos, endpos = tr.HitPos + Vector(0,0,-10000), filter=filt})
	end
	if not tr.Hit then return true end

	if tr.Entity:IsPlayer() then
		filt[#filt+1] = tr.Entity
		local tr2 = util.TraceLine({start = tr.HitPos, endpos = tr.HitPos + Vector(0,0,-10000), filter=filt, mask=MASK_SOLID})
		if tr2.Hit then
			tr.HitPos = tr2.HitPos
		end
	end

	if util.PointContents(tr.HitPos) & CONTENTS_WATER == 32 then return true end

	local ent = ents.Create("env_fire")
	if ent:IsValid() then
		ent:SetPos(tr.HitPos)
		ent:SetKeyValue("health", 10)
		ent:SetKeyValue("firesize", 32)
		ent:SetKeyValue("damagescale", -10)
		ent:SetKeyValue("spawnflags", "2")
		ent:SetOwner(pl)
		ent:Spawn()
		ent:Fire("Enable", "", 0)
		ent:Fire("StartFire", "", 0.1)
		ent:Fire("kill", "", 11)
		ent:SetTeamID(pl:Team())
		local timername = "burn"..ent:EntIndex()
		timer.Create(timername, 0.1, 0, BurnTimer, ent, timername, 3, 10)
		local effectdata = EffectData()
			effectdata:SetOrigin(tr.HitPos)
		util.Effect("burn", effectdata)
	end
end

function spells.StaticBall(pl)
	local ent = ents.Create("projectile_staticball")
	if ent:IsValid() then
		ent:SetPos(pl:GetShootPos())
		ent:SetOwner(pl)
		ent:Spawn()
		ent:SetTeamID(pl:Team())
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:EnableMotion(true)
			phys:EnableDrag(false)
			phys:EnableGravity(false)
			phys:SetBuoyancyRatio(0)
			phys:SetMass(5)
			phys:Wake()
			phys:ApplyForceCenter(pl:GetAimVector() * 4000)
		end

		pl:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)
	end
end

function spells.IceLance(pl)
	local proj = ents.Create("projectile_icelance")
	if proj:IsValid() then
		proj:SetPos(pl:GetShootPos())
		local dir = pl:GetAimVector()
		proj:SetAngles(dir:Angle())
		proj:SetOwner(pl)
		local col = team.GetColor(pl:Team())
		proj:SetColor(col.r, col.g, col.b, 120)
		proj.ProjectileHeading = dir
		proj:Spawn()
		proj:SetTeamID(pl:Team())
	end
end

local function CreateProtrusionSpike(pl, pos, teamid, fromweapon)
	local ent = ents.Create("projectile_protrusionspike")
	if ent:IsValid() then
		ent:SetPos(pos)
		if pl:IsValid() then
			ent:SetOwner(pl)
		end
		ent:SetTeamID(teamid)
		ent:Spawn()
		if fromweapon then
			ent.DamageMultiplier = pl.MeleeDamageMultiplier
		end
	end
end

function spells.Protrusion(pl, fromweapon)
	local vStart = pl:GetPos() + Vector(0,0,32)
	local vForward = pl:GetForward()
	local _filter = player.GetAll()
	table.Add(_filter, ents.FindByClass("projectile_protrusionspike"))

	local tocreate = {}

	local dist = 64
	--for i=1, 8 do
		local tr = util.TraceLine({start = vStart, endpos = dist * vForward + vStart, filter = _filter, mask = MASK_PLAYERSOLID})
		local tr2 = util.TraceLine({start = tr.HitPos, endpos = tr.HitPos + Vector(0, 0, -100), filter = _filter, mask = MASK_PLAYERSOLID})

		if tr2.Hit then
			--dist = dist + 64
			table.insert(tocreate, tr2.HitPos)
		end
	--end

	if #tocreate <= 0 then return true end

	if not fromweapon then
		pl:RestartGesture(ACT_SIGNAL_FORWARD)
	end

	local teamid = pl:Team()
	for i, pos in ipairs(tocreate) do
		timer.Simple(i * 0.06, CreateProtrusionSpike, pl, pos, teamid, fromweapon)
	end
end

function spells.AirHike(pl)
	if pl:IsCarrying() then return true end

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:GetPos() + Vector(0,0,2))
		effectdata:SetScale(40)
		effectdata:SetMagnitude(1.5)
		effectdata:SetRadius(40)
		effectdata:SetNormal(Vector(0,0,0))
	util.Effect("ThumperDust", effectdata)

	local vVel = pl:GetVelocity()
	pl:SetLocalVelocity(Vector(vVel.x, vVel.y, 400))
end

--[[local function ToxicCloudTimer(owner, team, pos, tn)
 	if not owner:IsValid() then
		owner = GetWorldEntity()
	end

	for _, ent in pairs(ents.FindInSphere(pos, 128)) do
		if not ent.PHealth and TrueVisible(pos, ent:NearestPoint(pos)) then
			if ent.TakeSpecialDamage then
				ent:TakeSpecialDamage(2, DMGTYPE_POISON, owner, DUMMY_TOXICCLOUD)
				if ent.Poison and ent:Alive() and ent:Team() ~= team then
					ent:Poison()
				end
			else
				ent:TakeDamage(2, owner, DUMMY_TOXICCLOUD)
			end
		end
	end
end]]

function spells.MissilesOfMagic(pl)
	local count = 0
	for _, ent in pairs(ents.FindByClass("projectile_magicmissile")) do
		if not ent.DontCount and ent:GetOwner() and ent:GetOwner() == pl then
			count = count + 1
		end
	end
	if 6 <= count then
		pl:LM(36)
		return true
	end

	pl:EmitSound("nox/missilesofmagic.wav")

	local created = 0
	local aimang = pl:GetAimVector():Angle()
	local one42 = 1 / 42
	for i=1, 6 do
		if 2 <= created or 6 <= count then return end
		local aimvec = aimang
		aimvec.yaw = aimvec.yaw + math.Rand(-15, 15)
		aimvec.pitch = aimvec.pitch + math.Rand(-5, 5)
		aimvec = aimvec:Forward()
		local ent = ents.Create("projectile_magicmissile")
		if ent:IsValid() then
			ent:SetPos(pl:GetShootPos())
			ent:SetOwner(pl)
			ent.Infravision = pl.Infravision
			ent:Spawn()
			ent:SetTeamID(pl:Team())
			local phys = ent:GetPhysicsObject()
			if phys:IsValid() then
				phys:Wake()
				phys:ApplyForceCenter(aimvec * 480)
			end
			created = created + 1
			count = count + 1
		end
	end

	pl:RestartGesture(ACT_SIGNAL_FORWARD)
end

function GenericHoming(pl, effect, friendly)
	local ent = ents.Create("projectile_generichoming")
	if ent:IsValid() then
		ent:SetPos(pl:GetShootPos())
		ent:SetOwner(pl)
		if friendly then
			ent.Friendly = true
			ent:SetSkin(1)
		end
		ent:Spawn()
		ent.Type = effect
		ent:SetTeamID(pl:Team())
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:ApplyForceCenter(pl:GetAimVector() * 750)
		end
		if pl.Infravision then
			local effectdata = EffectData()
				effectdata:SetEntity(ent)
				effectdata:SetOrigin(ent:EyePos())
				effectdata:SetMagnitude(10)
			util.Effect("infravision", effectdata, true)
			ent.Infravision = true
		end
		return ent
	end
end

function spells.DoMinorHeal(pl, generichit, owner)
	if not pl:IsValid() or not pl:Alive() then return end

	if pl:KeyDown(IN_USE) then
		GenericHoming(pl, "MinorHeal", true)
		return
	end

	GAMEMODE:PlayerHeal(pl, owner or pl, 6)

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:EyePos())
		effectdata:SetEntity(pl)
		effectdata:SetMagnitude(150)
		effectdata:SetScale(64)
	util.Effect("heal", effectdata)
end

function spells.MinorHeal(pl)
	if pl.HealCharge and pl.HealCharge:IsValid() then return true end

	pl:GiveStatus("healcharge_minor", 0.5)
end

function spells.DoLesserHeal(pl, generichit, owner)
	if not pl:IsValid() or not pl:Alive() then return end

	if pl:KeyDown(IN_USE) then
		GenericHoming(pl, "LesserHeal", true)
		return
	end

	GAMEMODE:PlayerHeal(pl, owner or pl, 12)

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:EyePos())
		effectdata:SetEntity(pl)
		effectdata:SetMagnitude(135)
		effectdata:SetScale(64)
	util.Effect("heal", effectdata)
end

function spells.LesserHeal(pl)
	if pl.HealCharge and pl.HealCharge:IsValid() then return true end

	pl:GiveStatus("healcharge_lesser", 0.75)
end

function spells.DoHeal(pl, generichit, owner)
	if not pl:IsValid() or not pl:Alive() then return end

	if pl:KeyDown(IN_USE) then
		GenericHoming(pl, "Heal", true)
		return
	end

	GAMEMODE:PlayerHeal(pl, owner or pl, 19)

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:EyePos())
		effectdata:SetEntity(pl)
		effectdata:SetMagnitude(115)
		effectdata:SetScale(64)
	util.Effect("heal", effectdata)
end

function spells.Heal(pl)
	if pl.HealCharge and pl.HealCharge:IsValid() then return true end

	pl:GiveStatus("healcharge", 1)
end

function spells.DoGreaterHeal(pl, generichit, owner)
	if not pl:IsValid() or not pl:Alive() then return end

	if pl:KeyDown(IN_USE) then
		GenericHoming(pl, "GreaterHeal", true)
		return
	end

	GAMEMODE:PlayerHeal(pl, owner or pl, 26)

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:LocalToWorld(pl:OBBCenter()))
	util.Effect("healringcomplete", effectdata)
end

function spells.GreaterHeal(pl)
	if pl.HealCharge and pl.HealCharge:IsValid() then return true end

	pl:GiveStatus("healcharge_greater", 1.25)
end

function spells.DoMajorHeal(pl, generichit, owner)
	if not pl:IsValid() or not pl:Alive() then return end

	if pl:KeyDown(IN_USE) then
		GenericHoming(pl, "GreaterHeal", true)
		return
	end

	GAMEMODE:PlayerHeal(pl, owner or pl, 35)

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:LocalToWorld(pl:OBBCenter()))
	util.Effect("healringcomplete", effectdata)
end

function spells.MajorHeal(pl)
	if pl.HealCharge and pl.HealCharge:IsValid() then return true end

	pl:GiveStatus("healcharge_major", 1.5)
end

function spells.SunBeam(pl)
	local pos = pl:TraceLine(1000).HitPos

	local effectdata = EffectData()
		effectdata:SetOrigin(pos)
		effectdata:SetStart(pl:GetShootPos())
	util.Effect("explosionbeam", effectdata)

	pl:RestartGesture(ACT_SIGNAL_HALT)

	timer.Simple(1, ExplosiveDamage, pl, pos, 115, 100, 0.9, 0.7, 16, DUMMY_SUNBEAM)
end

function spells.EnergyBolt(pl)
	if pl:InVehicle() then return true end

	if pl:IsMoving() or pl:GetMana() < 3 then return true end

	if pl.Lightning then
		pl:RemoveStatus("lightning", false, true)
	end

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:GetPos())
	util.Effect("energyboltstart", effectdata)

	pl:GiveStatus("energybolt")
end

function spells.Lightning(pl)
	if pl:InVehicle() then return true end

	if pl:IsMoving() or pl:GetMana() < 3 then return true end

	if pl.EnergyBolt then
		pl:RemoveStatus("energybolt", false, true)
	end

	local effectdata = EffectData()
		effectdata:SetOrigin(pl:GetPos())
	util.Effect("lightningstart", effectdata)

	pl:GiveStatus("lightning")
end

function CounterSpellEffect(pl, pos, range)
	local myteam = pl:Team()
	local counteredsomething = false
	for _, ent in pairs(ents.FindInSphere(pos, range or 450)) do
		if ent.CounterSpell then
			local owner = ent
			local towner = ent:GetOwner()
			if towner:IsPlayer() then
				owner = towner
			end
			if (ent:GetTeamID() ~= myteam or pl == owner) and TrueVisible(ent:NearestPoint(pos), pos)  then
				counteredsomething = true
				local effectdata = EffectData()
					effectdata:SetOrigin(ent:GetPos())
				util.Effect("counterspell", effectdata, true)
				if ent.CounterSpell == COUNTERSPELL_CUSTOM then
					ent:CounterSpelled(pl)
				elseif ent.CounterSpell == COUNTERSPELL_DESTROY then
					ent:Remove()
				elseif ent.CounterSpell == COUNTERSPELL_EXPLODE then
					ent:SetOwner(pl)
					ent:SetTeamID(myteam)
					ent:Explode()
				end
			end
		elseif ent:IsPlayer() and TrueVisible(ent:NearestPoint(pos), pos) then
			ent.CounterSpelled = CurTime() + 0.2
		end
	end
	if not counteredsomething then
		WorldSound("nox/counterspell.wav", pos, 90, math.random(95, 105))
	end
end

function spells.CounterSpell(pl)
	CounterSpellEffect(pl, pl:EyePos())

	pl:RestartGesture(ACT_SIGNAL_HALT)
end

function spells.Shock(pl)
	local mypos = pl:LocalToWorld(pl:OBBCenter())
	local myteam = pl:GetTeamID()
	for _, ent in pairs(ents.FindInSphere(mypos, 64)) do
		local nearest = ent:NearestPoint(mypos)
		if ent ~= pl and TrueVisible(nearest, mypos) then
			ent:TakeSpecialDamage(20, DMGTYPE_LIGHTNING, pl, DUMMY_SHOCK)
			if ent:IsPlayer() and ent:Alive() and ent:GetTeamID() ~= myteam then
				ent:SetGroundEntity(NULL)
				ent:SetVelocity((ent:LocalToWorld(ent:OBBCenter()) - nearest):Normalize() * 400 + Vector(0,0,300))
				ent:KnockDown()
			end
		end
	end

	local effectdata = EffectData()
		effectdata:SetOrigin(mypos)
	util.Effect("shock", effectdata, true, true)

	local effect2 = ents.Create("point_tesla")
	if effect2:IsValid() then
		effect2:SetKeyValue("m_flRadius", "150")
		effect2:SetKeyValue("m_SoundName", "DoSpark")
		effect2:SetKeyValue("m_Color", "255 255 255")
		effect2:SetKeyValue("texture", "effects/laser1.vmt")
		effect2:SetKeyValue("beamcount_min", "18")
		effect2:SetKeyValue("beamcount_max", "25")
		effect2:SetKeyValue("thick_min", "5")
		effect2:SetKeyValue("thick_max", "10")
		effect2:SetKeyValue("lifetime_min", "0.25")
		effect2:SetKeyValue("lifetime_max", "0.4")
		effect2:SetKeyValue("interval_min", "0.05")
		effect2:SetKeyValue("interval_max", "0.12")
		effect2:SetPos(mypos)
		effect2:Spawn()
		effect2:Fire("DoSpark", "", 0.1)
		effect2:Fire("DoSpark", "", 0.3)
		effect2:Fire("DoSpark", "", 0.5)
		effect2:Fire("kill", "", 0.8)
	end
end

function spells.ProtectFromFire(pl, override)
	if pl:KeyDown(IN_USE) and not override then
		GenericHoming(pl, "ProtectFromFire", true)
		return
	end

	pl:GiveStatus("protectorbs_fire"):SetOrbs(2)
end

function spells.ProtectFromCold(pl, override)
	if pl:KeyDown(IN_USE) and not override then
		GenericHoming(pl, "ProtectFromCold", true)
		return
	end

	pl:GiveStatus("protectorbs_cold"):SetOrbs(2)
end

function spells.ProtectFromShock(pl, override)
	if pl:KeyDown(IN_USE) and not override then
		GenericHoming(pl, "ProtectFromShock", true)
		return
	end

	pl:GiveStatus("protectorbs_shock"):SetOrbs(2)
end

function spells.Slow(pl)
	local ent = ents.Create("projectile_slow")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetPos(pl:GetShootPos())
		ent:SetTeamID(pl:Team())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 900)
		end
	end
end

function spells.Run(pl)
	local ent = ents.Create("projectile_run")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetPos(pl:GetShootPos())
		ent:SetTeamID(pl:Team())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 900)
		end
	end
end

function spells.ManaBurn(pl)
	local ent = ents.Create("projectile_manaburn")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetPos(pl:GetShootPos())
		ent:SetTeamID(pl:Team())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 900)
		end
	end
end

function spells.Anchor(pl)
	local ent = ents.Create("projectile_anchor")
	if ent:IsValid() then
		ent:SetOwner(pl)
		ent:SetPos(pl:GetShootPos())
		ent:SetTeamID(pl:Team())
		ent:Spawn()
		local phys = ent:GetPhysicsObject()
		if phys:IsValid() then
			phys:SetVelocityInstantaneous(pl:GetAimVector() * 900)
		end
	end
end

function spells.Tag(pl)
	GenericHoming(pl, "Tag")
end

function spells.CurePoison(pl, override)
	if pl:KeyDown(IN_USE) and not override then
		GenericHoming(pl, "CurePoison", true)
		return
	end

	pl:CurePoison()
end

function spells.Poison(pl)
	GenericHoming(pl, "Poison")
end

function spells.ForceField(pl, override)
	if pl:KeyDown(IN_USE) and not override then
		GenericHoming(pl, "ForceField", true)
		return
	end

	pl:GiveStatus("forcefield", 25)
	--pl:RestartGesture(ACT_SIGNAL_GROUP)
end

function CreateTeleportCloud(pos1, pos2, pl)
	local ent = ents.Create("teleportcloud")
	if ent:IsValid() then
		ent:SetPos(pos1)
		ent.TargetPos = pos2
		ent.Target = ent2
		if pl then
			ent.Owner = pl
			ent:SetTeamID(pl:Team())
		else
			ent.Owner = ent
			ent:SetTeamID(0)
		end
		ent:Spawn()
		ent:Think()
	end

	local effectdata = EffectData()
		effectdata:SetOrigin(pos1)
	util.Effect("teleportsparksstart", effectdata, true)
end

function spells.Inversion(pl)
	local myteam = pl:Team()
	local pos = pl:GetPos() + Vector(0,0,40)
	local invertedsomething = false
	for _, ent in pairs(ents.FindInSphere(pos, 390)) do
		if ent.Inversion and ent:GetTeamID() ~= myteam and IsVisible(ent:GetPos(), pos) then
			invertedsomething = true
			Invert(ent, pl)
		end
	end
	if invertedsomething then
		WorldSound("nox/inversion.wav", pos, 80, math.Rand(95, 105))

		pl:RestartGesture(ACT_SIGNAL_FORWARD)
	end
end

function Invert(ent, pl)
	if ent.Inversion == INVERSION_CUSTOM then
		ent:Inverted(pl)
	elseif ent.Inversion == INVERSION_DESTROY then
		local effectdata = EffectData()
			effectdata:SetOrigin(ent:GetPos())
		util.Effect("counterspell", effectdata, true)
		ent:Remove()
	elseif ent.Inversion == INVERSION_SETOWNER then
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		local effectdata = EffectData()
			effectdata:SetOrigin(ent:GetPos())
			effectdata:SetNormal((ent:GetPos() - pl:EyePos()):Normalize())
			effectdata:SetMagnitude(0)
		util.Effect("inversion", effectdata)
	elseif ent.Inversion == INVERSION_SETOWNER_REVERSE then
		ent:SetOwner(pl)
		ent:GetPhysicsObject():SetVelocityInstantaneous(ent:GetVelocity() * -1)
		ent:SetTeamID(pl:Team())
		local effectdata = EffectData()
			effectdata:SetOrigin(ent:GetPos())
			effectdata:SetNormal((ent:GetPos() - pl:EyePos()):Normalize())
			effectdata:SetMagnitude(0)
		util.Effect("inversion", effectdata)
	elseif ent.Inversion == INVERSION_SETOWNER_REVERSE_TARGET then
		ent.Target = ent:GetOwner()
		ent:SetOwner(pl)
		ent:SetTeamID(pl:Team())
		local effectdata = EffectData()
			effectdata:SetOrigin(ent:GetPos())
			effectdata:SetNormal((ent:GetPos() - pl:EyePos()):Normalize())
			effectdata:SetMagnitude(0)
		util.Effect("inversion", effectdata)
	elseif ent.Inversion == INVERSION_SETOWNER_REVERSE_TARGET_VELOCITY then
		ent.Target = ent:GetOwner()
		ent:SetOwner(pl)
		ent:GetPhysicsObject():SetVelocityInstantaneous(ent:GetVelocity() * -1)
		ent:SetTeamID(pl:Team())
		local effectdata = EffectData()
			effectdata:SetOrigin(ent:GetPos())
			effectdata:SetNormal((ent:GetPos() - pl:EyePos()):Normalize())
			effectdata:SetMagnitude(0)
		util.Effect("inversion", effectdata)
	end
end

function spells.ReflectiveShield(pl)
	if pl:IsMoving() then return true end

	pl:GiveStatus("reflectiveshield", 3)
end

GenericHit = {}

function Slow(pl, tim)
	pl:GiveStatus("slow", tim):SetColor(tim, 255, 255, 255)
end

function GenericHit.Explosion(pl, proj)
	pl:GiveStatus("explosion", 2).Attacker = proj:GetOwner()
end

function GenericHit.Tag(pl, proj)
	local owner = proj:GetOwner()
	if owner:IsValid() then
		pl:GiveStatus("tag", 10):SetColor(owner:EntIndex(), 255, 255, 255)
	end
end

function GenericHit.Stun(pl, proj)
	pl:Stun(2.5)
	local owner = proj:GetOwner()
	if owner:IsValid() then
		pl:SetLastAttacker(owner)
	end
end

function GenericHit.MinorHeal(pl, ent)
	spells.DoMinorHeal(pl, true, ent:GetOwner())
end

function GenericHit.LesserHeal(pl, ent)
	spells.LesserHeal(pl, true, ent:GetOwner())
end

function GenericHit.Heal(pl, ent)
	spells.Heal(pl, true, ent:GetOwner())
end

function GenericHit.GreaterHeal(pl, ent)
	spells.DoGreaterHeal(pl, true, ent:GetOwner())
end

function GenericHit.MajorHeal(pl, ent)
	spells.MajorHeal(pl, true, ent:GetOwner())
end

function GenericHit.ForceField(pl)
	spells.ForceField(pl, true)
end

function GenericHit.Poison(pl, proj)
	pl:Poison()
	local owner = proj:GetOwner()
	if owner:IsValid() then
		pl:SetLastAttacker(owner)
	end
end

util.PrecacheSound("player/pl_pain5.wav")
util.PrecacheSound("player/pl_pain6.wav")
util.PrecacheSound("player/pl_pain7.wav")
function DoPoison(ent, timername)
	if ent:IsValid() and ent:Alive() then
		local damage = 3

		ent.PoisonCount = ent.PoisonCount or 8
		ent.PoisonCount = ent.PoisonCount - 1

		if ent.ProtectFromPoison then
			if math.random(1, 6) == 1 then
				timer.Destroy(timername)
				return
			end
			damage = math.floor(damage * 0.666)
		end

		ent:EmitSound("player/pl_pain"..math.random(5, 7)..".wav")
		ent:SendLua("PoisEff()")

		if damage < ent:Health() then
			ent:SetHealth(ent:Health() - damage)
			if ent.PoisonCount <= 0 then
				timer.Destroy(timername)
			end
		else
			ent:SetHealth(1)
			timer.Destroy(timername)
		end
	else
		timer.Destroy(timername)
	end
end

function GenericHit.Haste(pl)
	spells.Haste(pl, true)
end

function GenericHit.Infravision(pl)
	spells.Infravision(pl, true)
end

function GenericHit.ProtectFromShock(pl)
	spells.ProtectFromShock(pl, true)
end

function GenericHit.ProtectFromFire(pl)
	spells.ProtectFromFire(pl, true)
end

function GenericHit.ProtectFromCold(pl)
	spells.ProtectFromCold(pl, true)
end

function GenericHit.ProtectFromPoison(pl)
	spells.ProtectFromPoison(pl, true)
end

function GenericHit.Vampirism(pl)
	spells.Vampirism(pl, true)
end

function GenericHit.Shock(pl)
	spells.Shock(pl, true)
end

function GenericHit.Regeneration(pl, ent)
	spells.Regeneration(pl, true, ent:GetOwner())
end

function GenericHit.SwapLocation(pl, proj)
	if GAMESTATE and GAMESTATE ~= GAMESTATE_PLAY then return end
	local owner = proj:GetOwner()
	if not owner.SendLua then return end
	if pl:IsCarrying() or owner:IsCarrying() or pl:InVehicle() or owner:InVehicle() or not owner:Alive() then return end

	if TrueVisible(owner:NearestPoint(pl:EyePos()), pl:NearestPoint(owner:EyePos())) then
		pl:SetLastAttacker(owner)
		local ownerpos = owner:GetPos()
		owner:SetPos(owner:GetPos() + Vector(0,0,72))
		local plpos = pl:GetPos()
		pl:SetPos(ownerpos)
		owner:SetPos(plpos)
		pl:EmitSound("nox/genericprojectile.wav")
	end
end

function ExplosiveDamage(attacker, inflictor, from, maxrange, minrange, distmultiplier, dmgmultiplier, mindamage, damagetype)
	local mypos

	--[[if type(from) == "Vector" then
		mypos = from + Vector(0,0,2)
	else
		mypos = from:GetPos() + Vector(0,0,2)
	end]]

	if from then
		mypos = from
	elseif inflictor then
		mypos = inflictor:LocalToWorld(inflictor:OBBCenter())
	else
		mypos = attacker:LocalToWorld(attacker:OBBCenter())
	end

	damagetype = damagetype or DMGTYPE_FIRE

	local damagedents = {}
	for _, ent in pairs(ents.FindInSphere(mypos, maxrange)) do
		local entpos = ent:NearestPoint(mypos)
		if TrueVisible(entpos, mypos) then
			if ent.TakeSpecialDamage then
				local basedam = math.max((minrange - entpos:Distance(mypos) * distmultiplier) * dmgmultiplier, mindamage)
				if damagetype == DMGTYPE_LIGHTNING then
					basedam = basedam + basedam * ent:WaterLevel() * 0.5
				end
				damagedents[ent] = basedam
				ent:TakeSpecialDamage(basedam, damagetype, attacker, inflictor or attacker)
			end
		end
	end

	util.ScreenShake(mypos, minrange * 17, minrange * 10, math.max(0.75, math.min(dmgmultiplier, 2)), maxrange * 2.5)

	return damagedents
end
