local meta = FindMetaTable("Entity")
if not meta then return end

function meta:Alive()
	return 0 < self:Health() 
end

function meta:IsInWater()
	return self:WaterLevel() > 0
end

function meta:IsSwimming()
	return self:WaterLevel() >= 3
end

function meta:IsSubmerged()
	return self:WaterLevel() >= 4
end

function meta:IsProjectile()
	return string.sub(self:GetClass(), 1, 11) == "projectile_"
end

function meta:SetTeamID(int)
	self.TeamID = int
end

function meta:GetTeamID()
	return self.TeamID or 0 -- or (self:EntIndex() + 2001)
end

function meta:IsInBox(vecmin, vecmax)
	for _, ent in pairs(ents.FindInBox(vecmin, vecmax)) do
		if self == ent then return true end
	end

	return false
end

function meta:IsInFlagArea()
	for _, i in pairs(TEAMS_PLAYING) do
		if team.TeamInfo[i] then
			local flagpoint = team.TeamInfo[i].FlagPoint
			if flagpoint and (GAMESTATE ~= GAMESTATE_PLAY and self:IsInBox(flagpoint + Vector(-90, -90, -90), flagpoint + Vector(90, 90, 90)) or GAMESTATE == GAMESTATE_PLAY and self:IsInBox(flagpoint + Vector(-12, -12, -90), flagpoint + Vector(12, 12, 90))) then
				return true
			end
		end
	end

	return false
end

function meta:GetVisibility()
	local r, g, b, a = self:GetColor()

	return a
end

function meta:FireOutput(outpt, activator, caller)
	local intab = self[outpt]
	if intab then
		for key, tab in pairs(intab) do
			for __, subent in pairs(ents.FindByName(tab.entityname)) do
				if tab.delay == 0 then
					subent:Input(tab.input, activator, caller, tab.args)
				else
					timer.Simple(tab.delay, function() if subent:IsValid() then subent:Input(tab.input, activator, caller, tab.args) end end)
				end
			end
		end
	end
end

function meta:AddOnOutput(key, value)
	self[key] = self[key] or {}
	local tab = string.Explode(",", value)
	table.insert(self[key], {entityname=tab[1], input=tab[2], args=tab[3], delay=tab[4], reps=tab[5]})
end

function meta:TakeSpecialDamage(damage, damagetype, attacker, inflictor, hitpos)
	attacker = attacker or self
	if not attacker:IsValid() then attacker = self end
	inflictor = inflictor or attacker
	if not inflictor:IsValid() then inflictor = attacker end

	local dmginfo = DamageInfo()
	dmginfo:SetDamage(damage)
	dmginfo:SetAttacker(attacker)
	dmginfo:SetInflictor(inflictor)
	dmginfo:SetDamagePosition(hitpos or self:NearestPoint(inflictor:NearestPoint(self:LocalToWorld(self:OBBCenter()))))
	dmginfo:SetDamageType(damagetype)
	self:TakeDamageInfo(dmginfo)
end

if SERVER then
	function meta:FireBullet(attacker, inflictor, pos, heading, speed, damage, class, callback)
		attacker = attacker or self
		inflictor = inflictor or attacker or self

		local bullet = ents.Create(class or "projectile_bullet")
		if bullet:IsValid() then
			bullet:SetPos(pos)
			bullet:SetAngles(heading:Angle())
			bullet:SetOwner(self)
			local teamid = attacker:GetTeamID()
			local col = team.GetColor(teamid) or color_white
			bullet:SetColor(col.r, col.g, col.b, 255)
			bullet:Spawn()
			bullet:SetTeamID(teamid)
			bullet.ProjectileHeading = heading
			bullet.Attacker = attacker
			bullet.Inflictor = inflictor
			bullet.ProjectileSpeed = speed
			bullet.Damage = damage
			bullet.Callback = callback
			bullet:GetPhysicsObject():SetVelocityInstantaneous(heading * speed)
		end
	end
end

if CLIENT then
	function meta:FireBullet()
	end

	--[[AR = 0
	AU = 0
	AF = 0
	RR = 90
	RU = -90
	RF = 0
	WID = 600
	HEI = 600
	function meta:DrawBasicPropInfo(pos, ang, title, titlecolor, ...)
		if not pos then
			ang = EYETRACE_HITNORMAL:Angle()
			ang.yaw = EyeAngles().yaw
			pos = self:NearestPoint(EYETRACE_HITPOS) + EYETRACE_HITNORMAL * 8

			ang:RotateAroundAxis(ang:Right(), RR)
			ang:RotateAroundAxis(ang:Up(), RU)
			ang:RotateAroundAxis(ang:Forward(), RF)
		end

		cam.Start3D2D(pos + ang:Right() * AR + ang:Up() * AU + ang:Forward() * AF, ang, 0.05)
			local wid, hei = WID, HEI
			surface.SetDrawColor(255, 255, 255, 255)
			surface.DrawRect(0, 0, wid, hei)
			surface.SetDrawColor(255, 0, 0, 255)
			surface.DrawLine(0, hei * 0.5, wid, hei * 0.5)
			surface.DrawLine(wid * 0.5, 0, wid * 0.5, hei)
			local y = 0
			if title then
				surface.DrawText(title, "DefaultBold", 0, 0, titlecolor, TEXT_ALIGN_LEFT)
				local tw, th = surface.GetTextSize(title)
				y = y + th
			end
			if arg[1] then
				for i=1, #arg, 4 do
					surface.DrawText(arg[i], "DefaultSmall", 0, y, arg[i + 3], TEXT_ALIGN_LEFT)
					local tw, th = surface.GetTextSize(arg[i])
					y = y + th
				end
			end
		cam.End3D2D()
	end]]
end
