local meta = FindMetaTable("Player")
if not meta then return end

function meta:Think(deltatime)
	if self:IsSprinting() then
		self:ThinkSprinting(deltatime)
	end
end

function meta:IsUsingSpecialMoves()
	return self:IsSprinting()
end

function meta:StopAllSpecialMoves()
	self:StopSprinting()
end

function meta:MoveSprinting(move)
	local delta = math.min(1, (CurTime() - self:GetSprintingStartTime()) * 2)
	local speed = move:GetMaxSpeed() + delta * 100 -- TODO: This is just a placeholder variable.
	move:SetMaxSpeed(speed)
	move:SetMaxClientSpeed(speed)
	move:SetForwardSpeed(speed)
	move:SetSideSpeed(move:GetSideSpeed() * (1 - delta))
end

function meta:ThinkSprinting(deltatime)
	if self:Crouching() or self:IsFrozen() or self:GetStamina() <= 0.1 then
		self:StopSprinting()
	else
		self:SetStamina(self:GetStamina() - (self.StaminaRegenerate + STAMINA_SPRINT) * deltatime) -- Sprinting negates stamina gain AND gives a stamina drain.
	end
end

function meta:StoppedSprinting()
	self:UpdateStamina()
end

function meta:StartedSprinting()
	self:UpdateStamina()
end

function meta:IsSprinting()
	return self:GetSprintingStartTime() > 0
end
meta.GetSprinting = meta.IsSprinting

function meta:GetSprintingStartTime()
	return self:GetDTFloat(0)
end

function meta:StopSprinting()
	if self:IsSprinting() then
		self:SetSprintingStartTime(0)

		self:StoppedSprinting()
	end
end

function meta:StartSprinting()
	if not self:IsSprinting() then
		self:SetSprintingStartTime(CurTime())

		self:StartedSprinting()
	end
end

function meta:SetSprintingStartTime(tim)
	self:SetDTFloat(0, tim)
end

function meta:GetTeamName()
	return team.GetName(self:Team()) or "None"
end

function meta:IsFacing(ent)
	local pos = ent:GetPos()
	local shootpos = self:GetShootPos()

	return pos:Distance(shootpos + self:GetAimVector() * 8) < pos:Distance(shootpos)
end

function meta:IsPointingAt(ent)
	return self:GetEyeTrace().Entity == ent
end

function meta:SetSpeed(speed)
	self:SetWalkSpeed(speed)
	self:SetRunSpeed(speed)
	self:SetMaxSpeed(speed)

	if speed == 1 or self.Stunned or self.HealRing or self.ChargeFireBall then
		self:SetJumpPower(0)
	else
		self:SetJumpPower(JUMPPOWER_DEFAULT * self.AgilityMultiplier)
	end
end

function meta:ReleasedForward()
	self:StopSprinting()
end

function meta:PressedForward()
	--local systime = os.clock()
	local systime = UnPredictedCurTime()
	if systime == self.m_LastKeyPressedForward then return end -- Bug on client-side makes it so KeyPress is called a shit load of times on the same frame.

	if systime <= self.m_LastKeyPressedForward + 0.3 then -- TODO: Maybe have this as a replicated convar instead of 0.3?
		self:PressedForwardTwice()
	else
		self.m_LastKeyPressedForward = systime
	end
end

function meta:PressedJump()
	if not self:Alive() then return end
	if CLIENT then return end

	local onground = self:IsOnGround()

	local stopsprint = true
	if onground then
		if self.Stunned or self.Anchored then
			self.NoJump = 0
			self:SetJumpPower(0)
			stopsprint = false
		elseif self.NoJump then
			if self.NoJump <= CurTime() then
				self.NoJump = nil
				--[[if self:GetStamina() <= 1 then
					self:SetJumpPower(250)
				else]]
					self:SetJumpPower(JUMPPOWER_DEFAULT * self.AgilityMultiplier)
				--end
			else
				self:SetJumpPower(0)
				stopsprint = false
			end
		end
	end

	if stopsprint and self.Sprinting then
		if onground and not self.VelocityRecovery then
			local vel = self:GetVelocity()
			if self:GetMaxSpeed() < vel:Length() then
				if self:GetStamina() < STAMINA_LONGJUMP then
					self:SetLocalVelocity(vel:Normalize() * self:GetMaxSpeed() + Vector(0,0,vel.z))
				else
					self:SetStamina(self:GetStamina() - STAMINA_LONGJUMP)
				end
			end
		end
		self:RemoveStatus("sprint", false, true)
	end

	if self:CanWallJump() then
		self.CantWallJump = CurTime() + 0.2
		if self.DashDodging then
			self:RemoveStatus("dashdodge*", false, true)
		end
		if self.Sprinting then
			self:RemoveStatus("sprint", false, true)
		end
		if self.WallRunning then
			self:RemoveStatus("wallrun*", false, true)
		end
		local getpos = self:GetPos()
		local forward = self:GetForward()
		local tr = util.TraceLine({start = getpos, endpos = getpos + forward * 36, mask = MASK_SOLID, filter = pl})
		local status = self:GiveStatus("walljump", 0.1)
		status.HitNormal = (2 * tr.HitNormal * tr.HitNormal:Dot(forward * -1) + forward):Normalize()
		status.Forward = forward
		self:SetStamina(self:GetStamina() - STAMINA_WALLJUMP, true)
	end
end

function meta:PressedForwardTwice()
	if self:CanWallRun() then
		local getpos = self:GetPos() + Vector(0, 0, 4)
		if -200 <= self:GetVelocity().z then
			local fwdend = self:GetForward() * 24
			local trup = util.TraceLine({start = getpos, endpos = getpos + fwdend, mask = MASK_PLAYERSOLID, filter = self})
			local getpos2 = getpos + Vector(0, 0, self:GetStepSize())
			local trup2 = util.TraceLine({start = getpos2, endpos = getpos2 + fwdend, mask = MASK_PLAYERSOLID, filter = self})
			if trup.Hit and not trup.HitSky and math.abs(trup.HitNormal.z) <= 0.18 and trup2.Hit and not trup2.HitSky and math.abs(trup2.HitNormal.z) <= 0.18 then
				self.m_LastKeyPressedForward = 0
				self:GiveStatus("wallrunup")

				return true
			end
		end

		if not self:IsOnGround() then
			local right = self:GetRight() * 24
			local tr = util.TraceLine({start = getpos, endpos = getpos + right, mask = MASK_PLAYERSOLID, filter = self})
			local tr2 = util.TraceLine({start = getpos, endpos = getpos - right, mask = MASK_PLAYERSOLID, filter = self})

			if tr.Hit and not tr.HitSky and math.abs(tr.HitNormal.z) <= 0.18 then
				self.m_LastKeyPressedForward = 0
				self:GiveStatus("wallrunright")

				return true
			elseif tr2.Hit and not tr2.HitSky and math.abs(tr2.HitNormal.z) <= 0.18 then
				self.m_LastKeyPressedForward = 0
				self:GiveStatus("wallrunleft")

				return true
			end
		end
	end

	if self:CanSprint() then
		self.m_LastKeyPressedForward = 0
		--self:GiveStatus("sprint")
		self:StartSprinting()

		return true
	end
end

function meta:Stun(tim, noeffect)
	if not self:InVehicle() then
		if noeffect then
			if self.Stunned and not self.SilentStunned then
				self:GiveStatus("stun", tim):SetColor(tim, 255, 255, 255)
			else
				self:GiveStatus("stun_noeffect", tim):SetColor(tim, 255, 255, 255)
			end
		else
			if self.SilentStunned then
				self:RemoveStatus("stun_noeffect", true, true)
			end
			self:GiveStatus("stun", tim):SetColor(tim, 255, 255, 255)
		end
	end
end

function meta:IsMoving()
	return self:KeyDown(IN_FORWARD) or self:KeyDown(IN_MOVELEFT) or self:KeyDown(IN_MOVERIGHT) or self:KeyDown(IN_BACK) or not self:IsOnGround()
end

function meta:ManaStun(tim, force)
	if force or 0 < self.MaxMana then
		self:GiveStatus("manastun", tim):SetColor(tim, 255, 255, 255)
	end
end

function meta:CanEnterVehicle()
	return self:Team() ~= TEAM_SPECTATE and not self.Stunned and GAMESTATE == GAMESTATE_PLAY and self:CanSwitchWeapon()
end

function meta:CanSwitchWeapon()
	local wep = self:GetActiveWeapon()
	if wep.CanSwitchWeapon and not wep:CanSwitchWeapon(towep) then
		return false
	end

	return true
end

function meta:SetNextAttack(tim)
	self.m_NextAttack = tim
end

function meta:GetNextAttack()
	return self.m_NextAttack
end

function meta:AirBrake(z)
	if not self:IsOnGround() then
		local curvel = self:GetVelocity()
		self:SetLocalVelocity(Vector(curvel.x * 0.2, curvel.y * 0.2, z or (curvel.z * 0.2)))

		return true
	end
end

function meta:TraceLine(distance, _mask)
	local vStart = self:GetShootPos()
	local filt = ents.FindByClass("projectile_*")
	table.insert(filt, self)
	return util.TraceLine({start = vStart, endpos = vStart + self:GetAimVector() * distance, filter = filt, mask = _mask})
end

function meta:ConvertNet()
	return ConvertNet(self:SteamID())
end

function meta:AddFrozenPhysicsObject(ent, phys)
end

function meta:UnfreezePhysicsObjects(ent, phys)
	return 0
end

function meta:GetMana()
	return math.min(self.MaxMana, self.Mana + self.ManaRegenerate * (CurTime() - self.ManaBase))
end

function meta:GetStamina()
	return math.min(self.MaxStamina, self.Stamina + self.StaminaRegenerate * (CurTime() - self.StaminaBase))
end

meta.GetTeamID = meta.Team

function meta:CanSprint()
	if not self.AbleToSprint or self.CantSprintTime and CurTime() <= self.CantSprintTime then
		return false
	end

	if self:GetStamina() <= 1 then return false end

	local wep = self:GetActiveWeapon()
	if not wep.CanSprint or wep:CanSprint(self) then
		return not self.Stunned and not self.HealCharge and self:WaterLevel() < 2 and not self:InVehicle() and self:GetMoveType() ~= MOVETYPE_LADDER and not self:IsFrozen() and not self.Sprinting and not self:Crouching() and not self.DashDodging and not self.WallJumping and not self.WallRunning
	end
end

function meta:CanWallRun()
	if not self.AbleToWallRun or self.CantSprintTime and CurTime() <= self.CantSprintTime then
		return false
	end

	if self:GetStamina() <= 1 then return false end

	local wep = self:GetActiveWeapon()
	if not wep.CanSprint or wep:CanSprint(self) then
		return not self.Stunned and not self.HealCharge and self:WaterLevel() < 2 and not self:InVehicle() and self:GetMoveType() ~= MOVETYPE_LADDER and not self:IsFrozen() and not self.Sprinting and not self:Crouching() and not self.DashDodging and not self.WallJumping and not self.WallRunning
	end
end

function meta:CanWallVert()
	if not self.AbleToWallRun or not self.AbleToWallVert or self.CantSprintTime and CurTime() <= self.CantSprintTime then
		return false
	end

	if self:GetStamina() <= 1 then return false end

	local wep = self:GetActiveWeapon()
	if not wep.CanSprint or wep:CanSprint(self) then
		return not self.Stunned and not self.HealCharge and self:WaterLevel() < 2 and not self:InVehicle() and self:GetMoveType() ~= MOVETYPE_LADDER and not self:IsFrozen() and not self.Sprinting and not self:Crouching() and not self.DashDodging and not self.WallJumping and not self.WallRunning
	end
end

function meta:CanHolster()
	return self:CanSwitchWeapon()
end

function meta:CanWallJump()
	if not self.AbleToWallJump or self:IsOnGround() or self.CantWallJumpTime and CurTime() <= self.CantWallJumpTime then
		return false
	end

	local wep = self:GetActiveWeapon()
	if not wep.CanWallJump or wep:CanWallJump(self) then
		if not self.HealCharge and self:WaterLevel() < 2 and not self:InVehicle() and self:GetMoveType() ~= MOVETYPE_LADDER and not self:IsFrozen() and self.NextWallJump <= CurTime() and not self.WallJumping and STAMINA_WALLJUMP <= self:GetStamina() then
			local tr = util.TraceLine({start = self:GetPos(), endpos = self:GetPos() + self:GetForward() * 36, mask = MASK_PLAYERSOLID, filter = self})
			return tr.Hit and not tr.HitSky and math.abs(tr.HitNormal.z) <= 0.19 
		end
	end
end

function meta:CanDodge()
	if not self.AbleToDodge or self.CantDodgeTime and CurTime() <= self.CantDodgeTime then
		return false
	end

	local wep = self:GetActiveWeapon()
	if not wep.CanDodge or wep:CanDodge(self) then
		return not self.HealCharge and self:WaterLevel() < 2 and self:GetMoveType() ~= MOVETYPE_LADDER and not self.DashDodging and not self.WallJumping and not self.IsReloading and not self.SilentStunned and not self.WallRunning and (not self:IsFrozen() or self.KnockedDown)
	end
end

function meta:IsCarrying()
	--[[if self.Carrying then
		if self.Carrying:GetPos():Distance(self:GetPos()) > 300 then
			if self.Carrying.Carrier == self then
				self.Carrying.Carrier = self.Carrying.Carrier
				self.Carrying:Returned(nil, true)
			end
			self.Carrying = nil
		end
	end]]

	return self.Carrying
end

if SERVER then
	function meta:UpdateStamina()
		umsg.Start("SLS", self)
			umsg.Float(self.Stamina)
			umsg.Float(self.StaminaBase)
		umsg.End()
	end

	function meta:UpdateMana()
		umsg.Start("SLM", self)
			umsg.Float(self.Mana)
			umsg.Float(self.ManaBase)
		umsg.End()
	end

	function meta:SetVelocityRecovery(onoff, nosend)
		self.VelocityRecovery = onoff
		if not nosend then
			self:SendLua("MySelf:SetVelocityRecovery("..tostring(onoff)..")")
		end
	end

	function meta:SetStamina(stamina, send)
		self.Stamina = stamina
		self.StaminaBase = CurTime()
		if send then
			self:UpdateStamina()
		end
	end

	function meta:SetMana(mana, send)
		self.Mana = mana
		self.ManaBase = CurTime()
		if send then
			self:UpdateMana()
		end
	end

	function meta:DI(spellid, ttime)
		umsg.Start("DI", self)
			umsg.Short(spellid)
			umsg.Float(ttime)
		umsg.End()
	end

	meta.OldSetMaxHealth = _R["Entity"].SetMaxHealth

	function meta:SetMaxHealth(num)
		if self:GetSkin() ~= num then
			self:SetSkin(num)
		end
		self:OldSetMaxHealth(num)
	end

	meta.OldExitVehicle = meta.ExitVehicle
	function meta:ExitVehicle()
		local veh = self:GetVehicle()
		if veh and veh:IsValid() then
			self:OldExitVehicle()

			local vehbase = veh:GetNetworkedEntity("vehicle", NULL)
			if vehbase.Exit then
				vehbase:Exit(pl, veh)
			end
		end
	end

	meta.OldGive = meta.Give
	function meta:Give(wep)
		self.AllowWeaponPickup = true
		self:OldGive(wep)
		self.AllowWeaponPickup = false
	end

	function meta:KnockDown(tim)
		if self:Alive() and not self:InVehicle() then
			self:GiveStatus("knockdown", tim or 5)

			self:HitReset(NULL, NULL, true)

			gamemode.Call("OnKnockedDown", self)
		end
	end

	if not meta.OldDrawViewModel then
		meta.OldDrawViewModel = meta.DrawViewModel
		meta.OldDrawWorldModel = meta.DrawWorldModel

		local function OldDrawViewModel(self, bDraw)
			if self:IsValid() then
				self:OldDrawViewModel(bDraw)
			end
		end

		function meta:DrawViewModel(bDraw)
			self.m_DrawViewModel = bDraw
			timer.Simple(0, OldDrawViewModel, self, bDraw)
		end

		local function OldDrawWorldModel(self, bDraw)
			if self:IsValid() then
				self:OldDrawWorldModel(bDraw)
			end
		end

		function meta:DrawWorldModel(bDraw)
			self.m_DrawWorldModel = bDraw
			timer.Simple(0, OldDrawWorldModel, self, bDraw)
		end

		function meta:GetDrawViewModel()
			return self.m_DrawViewModel
		end

		function meta:GetDrawWorldModel()
			return self.m_DrawWorldModel
		end
	end

	function meta:SetMaxMana(amount, regeneration)
		self.MaxMana = amount
		self.ManaRegenerate = regeneration
		self:SendLua("MySelf:SetMaxMana("..amount..","..regeneration..")")
	end

	function meta:SetMaxStamina(amount, regeneration)
		self.MaxStamina = amount
		self.StaminaRegenerate = regeneration
		self:SendLua("MySelf:SetMaxStamina("..amount..","..regeneration..")")
	end

	function meta:IsFrozen()
		return self.m_IsFrozen
	end

	meta.OldFreeze = meta.Freeze
	function meta:Freeze(bFreeze)
		if not bFreeze and GAMEMODE.Intermission and self ~= GAMEMODE.IntermissionPlayer then return end
		self.m_IsFrozen = bFreeze
		self:OldFreeze(bFreeze)
	end

	function meta:SetTeamID(iTeam)
		self:SetTeam(iTeam)
	end

	function meta:LMR(int, args)
		umsg.Start("lmr", self)
			umsg.Short(int)
			umsg.String(args or "")
		umsg.End()
	end

	function meta:LMG(int, args)
		umsg.Start("lmg", self)
			umsg.Short(int)
			umsg.String(args or "")
		umsg.End()
	end

	function meta:LM(int, args)
		umsg.Start("lm", self)
			umsg.Short(int)
			umsg.String(args or "")
		umsg.End()
	end

	function meta:sP()
		umsg.Start("sp", self)
		umsg.End()
	end

	function meta:SetLastAttacker(attacker)
		--[[if attacker ~= self.LastAttacker then
			self.LastAttacker2 = self.LastAttacker
			self.LastAttacked2 = self.LastAttacked
		end]]
		if attacker ~= self then
			self.LastAttacker = attacker
			self.LastAttacked = CurTime()
		end
	end

	function meta:GetLastAttacker()
		return self.LastAttacker, self.LastAttacked, self.LastAttacker2, self.LastAttacked2
	end

	function meta:ClearLastAttacker()
		self.LastAttacker = NULL
		self.LastAttacked = 0
		self.LastAttacker2 = NULL
		self.LastAttacked2 = 0
	end

	function meta:RemoveSummoning()
		if self.Summoning and self.Summoning:IsValid() then
			local effectdata = EffectData()
				effectdata:SetOrigin(self.Summoning:GetPos())
			util.Effect("counterspell", effectdata, true)
			self.Summoning:EmitSound("nox/summondie.wav")
			self.Summoning:Remove()
		end
	end

	function meta:RemoveAllStatus(bSilent, bInstant)
		if bInstant then
			for _, ent in pairs(ents.FindByClass("status_*")) do
				if not ent.NoRemoveOnDeath and ent:GetOwner() == self then
					ent:Remove()
				end
			end
		else
			for _, ent in pairs(ents.FindByClass("status_*")) do
				if not ent.NoRemoveOnDeath and ent:GetOwner() == self then
					ent.SilentRemove = bSilent
					ent:SetDie()
				end
			end
		end
	end

	function meta:RemoveStatus(sType, bSilent, bInstant)
		local removed
		for _, ent in pairs(ents.FindByClass("status_"..sType)) do
			if ent:GetOwner() == self then
				if bInstant then
					ent:Remove()
				else
					ent.SilentRemove = bSilent
					ent:SetDie()
				end
				removed = true
			end
		end
		return removed
	end

	function meta:GetStatus(sType)
		local ent = self["status_"..sType]
		if ent and ent.Owner == self then return ent end
	end

	function meta:GiveStatus(sType, fDie)
		local cur = self:GetStatus(sType)
		if cur then
			if fDie then
				cur:SetDie(fDie)
			end
			cur:SetPlayer(self, true)
			return cur
		else
			local ent = ents.Create("status_"..sType)
			if ent:IsValid() then
				ent:Spawn()
				if fDie then
					ent:SetDie(fDie)
				end
				ent:SetPlayer(self)
				return ent
			end
		end
	end

	function meta:TeamWeight()
		--return math.min(10000, (self.CTFKills / math.max(1, self.CTFDeaths)) * self.CTFKills + (self.ArenaWins / math.max(1, self.ArenaLosses)) * self.ArenaWins) * 0.65
		--return math.min(10000, (self.CTFKills / math.max(1, self.CTFDeaths)) * self.CTFKills + (self.AssaultWins / math.max(1, self.AssaultLosses)) * self.AssaultWins + self.AssaultDefense + self.AssaultOffense) * 0.65
		return math.min(8000, (self.CTFKills / math.max(1, self.CTFDeaths)) * self.CTFKills + (self.AssaultWins / math.max(1, self.AssaultLosses)) * self.AssaultWins * 50 + self.AssaultDefense + self.AssaultOffense) * 0.5
	end

	function meta:IsPoisoned()
		return timer.IsTimer(self:UniqueID().."Poisoned")
	end

	function meta:Poison()
		if not self:Alive() or self:IsPoisoned() or self.ProtectFromPoison then return end

		self.PoisonCount = 8
		local timername = self:UniqueID().."Poisoned"
		timer.Create(timername, 2, 0, DoPoison, self, timername)

		self:EmitSound("nox/poisonon.wav")

		self:DI(NameToSpell.Poison, 16)
	end
	GM:AddStatusTimer("Poisoned")

	function meta:CurePoison()
		self:DI(NameToSpell.Poison, 0)
		self:EmitSound("nox/heal.wav")

		timer.Destroy(self:UniqueID().."Poisoned")
	end

	--[[meta.OldTakeDamage = meta.TakeDamage
	function meta:TakeDamage(damage, attacker, inflictor)
		damage, attacker, inflictor = self:ProcessDamage(damage, attacker, inflictor)

		self:OldTakeDamage(damage, attacker, inflictor)
	end]]

	function meta:ProcessDamage(attacker, inflictor, dmginfo)
		StatusWeaponHook("ProcessDamage", self, attacker, inflictor, dmginfo)

		--[[if self:Health() <= damage then
			if self.ForceField then
				damage = 0
				self:RemoveStatus("forcefield", false, true)
				self.ForceField = nil
			else
				local dmgtype = self.LastDamageType
				if dmgtype == DMGTYPE_FIRE then
					local effectdata = EffectData()
						effectdata:SetOrigin(self:EyePos())
						effectdata:SetEntity(self)
					util.Effect("fire_death", effectdata, true, true)
					self.GibEffects = dmgtype
				elseif dmgtype == DMGTYPE_LIGHTNING then
					local effectdata = EffectData()
						effectdata:SetOrigin(self:EyePos())
						effectdata:SetEntity(self)
					util.Effect("electric_death", effectdata, true, true)
					self.GibEffects = dmgtype
				elseif dmgtype == DMGTYPE_ICE then
					local effectdata = EffectData()
						effectdata:SetOrigin(self:EyePos())
						effectdata:SetEntity(self)
					util.Effect("ice_death", effectdata, true, true)
					self.GibEffects = dmgtype
				end
			end
		end]]

		--[[if 0 < damage then
			if self.HealCharge and self.HealCharge:IsValid() then
				self.HealCharge.Interrupted = true
				self:RemoveStatus("healcharge*", false, true)
			end

			if self.ForceField then
				damage = damage * 0.75
			end
		end]]
	end

	--[[function meta:ProcessSpecialDamage(damage, dmgtype, attacker, inflictor)
		local prot
		if dmgtype == DMGTYPE_FIRE then
			prot = self.ProtectFromFire
		elseif dmgtype == DMGTYPE_LIGHTNING then
			prot = self.ProtectFromShock
		elseif dmgtype == DMGTYPE_POISON then
			prot = self.ProtectFromPoison
		elseif dmgtype == DMGTYPE_ICE then
			prot = self.ProtectFromCold
		end

		if prot and prot:IsValid() then
			if gamemode.Call("PlayerShouldTakeDamage", self, attacker) then
				if prot:GetSkin() == 1 then
					prot:Remove()
				else
					prot:SetSkin(prot:GetSkin() - 1)
				end
			end

			damage = damage * 0.4
		end

		return damage, dmgtype, attacker, inflictor
	end]]

	function meta:ForceRespawn()
		self:StripWeapons()
		if self.Invisible then
			timer.Destroy(self:UniqueID().."Invisible")
			local r, g, b, a = self:GetColor()
			self:SetColor(r, g, b, 255)
			self.Invisible = nil
		end
		self.LastDeath = CurTime()
		self:RemoveAllStatus(true, true)
		self:Spawn()
		self.SpawnTime = CurTime()
		self:sP()
	end

	function meta:BloodSpray(pos, num, dir, force)
		local effectdata = EffectData()
			effectdata:SetOrigin(pos)
			effectdata:SetMagnitude(num)
			effectdata:SetRadius(self.BloodDye or 0)
			effectdata:SetNormal(dir)
			effectdata:SetScale(force)
			effectdata:SetEntity(self)
		util.Effect("bloodstream", effectdata, true, true)
	end

	function meta:HitByMelee(attacker, attackerweapon, nostop)
		self:HitReset(attacker, attackerweapon, nostop)

		if not nostop then
			local vel = self:GetVelocity()
			self:SetLocalVelocity(vel * 0.05)
		end
	end

	function meta:HitReset(attacker, attackerweapon, nostop)
		attacker = attacker or NULL
		attackerweapon = attackerweapon or NULL

		local wep = self:GetActiveWeapon()
		if wep.HitByMelee and wep:HitByMelee(attacker, attackerweapon) then
			return
		end

		self:RestartGesture(ACT_HL2MP_GESTURE_RANGE_ATTACK_PHYSGUN)

		if self.Stunned then
			self:RemoveStatus("stun*", false, true)
		end
		if self.Sprinting then
			self:RemoveStatus("sprint", false, true)
		end
		if self.DashDodging then
			self:RemoveStatus("dashdodge*", false, true)
		end
		if self.WallRunning then
			self:RemoveStatus("dashdodge*", false, true)
		end

		local timnam = "DoSpellFor"..self:UniqueID()
		if timer.IsTimer(timnam) then
			self.Casting = nil
			timer.Destroy(timnam)
			umsg.Start("SIStop")
				umsg.Entity(sender)
			umsg.End()
		end
	end

	function meta:Gib(dmginfo)
		self.Gibbed = true
		local effectdata = EffectData()
			effectdata:SetEntity(self)
			effectdata:SetOrigin(self:EyePos())
			effectdata:SetScale(self.GibEffects)
			effectdata:SetRadius(self.BloodDye or 0)
		util.Effect("gib_player", effectdata, true, true)
	end

	function meta:DeathRayed()
		local effectdata = EffectData()
			effectdata:SetOrigin(self:EyePos())
			effectdata:SetEntity(self)
		util.Effect("DeathRayDeath", effectdata, true, true)
	end

	local OffenseAwards = {}
	OffenseAwards[30] = "Copper_Offense"
	OffenseAwards[60] = "Iron_Offense"
	OffenseAwards[100] = "Steel_Offense"
	OffenseAwards[150] = "Titanium_Offense"
	OffenseAwards[175] = "Diamond_Offense"
	OffenseAwards[250] = "Ultimate_Offense"

	function meta:AddOffense(amount)
		local newdef = self.OffenseThisRound + amount
		self.OffenseThisRound = newdef
		-- TODO: Change to new stats
		--self.AssaultOffense = self.AssaultOffense + amount

		self:SetNetworkedString("tpstats", self.KillsThisRound.."@"..self.AssistsThisRound.."@"..newdef.."@"..self.DefenseThisRound)

		for amount, award in pairs(OffenseAwards) do
			if amount <= newdef then
				if not self:HasAward(award) then
					NDB.GiveAward(self, award)
				end
			end
		end
	end

	local DefenseAwards = {}
	DefenseAwards[10] = "Copper_Defense"
	DefenseAwards[20] = "Iron_Defense"
	DefenseAwards[35] = "Steel_Defense"
	DefenseAwards[50] = "Titanium_Defense"
	DefenseAwards[75] = "Diamond_Defense"
	DefenseAwards[100] = "Ultimate_Defense"

	function meta:AddDefense(amount)
		local newdef = self.DefenseThisRound + amount
		self.DefenseThisRound = newdef
		-- TODO: Change to new stats
		--self.AssaultDefense = self.AssaultDefense + amount

		self:SetNetworkedString("tpstats", self.KillsThisRound.."@"..self.AssistsThisRound.."@"..self.OffenseThisRound.."@"..newdef)

		for amount, award in pairs(DefenseAwards) do
			if amount <= newdef then
				if not self:HasAward(award) then
					NDB.GiveAward(self, award)
				end
			end
		end
	end

	function meta:AddAssists(amount)
		local newdef = self.AssistsThisRound + amount
		self.AssistsThisRound = newdef
		-- TODO: Change to new stats
		--self.TeamPlayAssists = self.TeamPlayAssists + amount

		self:SetNetworkedString("tpstats", self.KillsThisRound.."@"..newdef.."@"..self.OffenseThisRound.."@"..self.DefenseThisRound)
	end

	function meta:AddKills(amount)
		local newdef = self.KillsThisRound + amount
		self.KillsThisRound = newdef
		-- TODO: Change to new stats
		--self.CTFKills = self.CTFKills + amount

		self:SetNetworkedString("tpstats", newdef.."@"..self.AssistsThisRound.."@"..self.OffenseThisRound.."@"..self.DefenseThisRound)
	end
end

if CLIENT then
	function meta:UpdateStamina()
	end

	function meta:UpdateMana()
	end

	function meta:SetVelocityRecovery(onoff)
		self.VelocityRecovery = onoff
	end

	function meta:SetMana(mana)
		self.Mana = mana
		self.ManaBase = CurTime()
	end

	function meta:SetStamina(stamina)
		self.Stamina = stamina
		self.StaminaBase = CurTime()
	end

	function meta:DI(spellid, ttime)
		if self == MySelf then
			DI(spellid, ttime)
		end
	end

	meta.OldGetMaxHealth = _R["Entity"].GetMaxHealth
	function meta:GetMaxHealth()
		return self:GetSkin()
	end

	function meta:KnockDown(tim)
		if not self:InVehicle() then
			self:GiveStatus("knockdown", tim or 5)

			gamemode.Call("OnKnockedDown", self)
		end
	end

	if not meta.SetGroundEntity then
		function meta:SetGroundEntity(ent)
		end
	end

	function meta:SetMaxMana(amount, regeneration)
		self.MaxMana = amount
		self.ManaRegenerate = regeneration
	end

	function meta:SetMaxStamina(amount, regeneration)
		self.MaxStamina = amount
		self.StaminaRegenerate = regeneration
	end

	function meta:GetMaxHealth()
		return 100 --Classes[self:GetNetworkedInt("class", 1)].Health
	end

	function meta:GetStatus(sType)
		for _, ent in pairs(ents.FindByClass("status_"..sType)) do
			if ent:GetOwner() == self then return ent end
		end
	end

	function meta:RemoveStatus()
	end
	meta.GiveStatus = meta.RemoveStatus
end

local function SortUserID(a, b)
	return a:EntIndex() < b:EntIndex()
end

function meta:GetNextSpectate()
	--local allplayers = player.GetAll()
	local allplayers = {}
	for i, teamid in pairs(TEAMS_PLAYING) do
		table.Add(allplayers, team.GetPlayers(teamid))
	end
	table.Add(allplayers, ents.FindByClass(GAMEMODE.FlagEntity))
	--[[for i, pl in ipairs(allplayers) do
		if pl == self then
			table.remove(allplayers, i)
			break
		end
	end]]
	table.sort(allplayers, SortUserID)

	local currentobs = self:GetObserverTarget()
	if currentobs and currentobs:IsValid() then
		for i=1, #allplayers do
			if allplayers[i] == currentobs then
				if allplayers[i + 1] then
					return allplayers[i + 1]
				end

				break
			end
		end
	end

	return allplayers[1]
end

function meta:GetPrevSpectate()
	--local allplayers = player.GetAll()
	local allplayers = {}
	for i, teamid in pairs(TEAMS_PLAYING) do
		table.Add(allplayers, team.GetPlayers(teamid))
	end
	table.Add(allplayers, ents.FindByClass(GAMEMODE.FlagEntity))
	--[[for i, pl in ipairs(allplayers) do
		if pl == self then
			table.remove(allplayers, i)
			break
		end
	end]]
	table.sort(allplayers, SortUserID)

	local currentobs = self:GetObserverTarget()
	if currentobs and currentobs:IsValid() then
		for i=#allplayers, 1, -1 do
			if allplayers[i] == currentobs then
				if allplayers[i - 1] then
					return allplayers[i - 1]
				end

				break
			end
		end
	end

	return allplayers[#allplayers]
end

meta.OldAlive = meta.Alive
function meta:Alive()
	if self:Team() == TEAM_SPECTATE then return false end

	return self:OldAlive()
end
