include("noxteams.lua")
include("obj_entity_extend.lua")
include("obj_player_extend.lua")
include("obj_weapon_extend.lua")
include("globals.lua")
include("animations.lua")

GM.Name = "Team Play 2"
GM.Author = "William \"JetBoom\" Moodhe"
GM.Email = "williammoodhe@gmail.com"
GM.Website = "www.noxiousnet.com"
GM.Credits = {"William \"JetBoom\" Moodhe", "http://www.noxiousnet.com (williammoodhe@gmail.com)", "Project Lead / Programmer",
"Zetanor", "http://www.zetanor.net ()", "Programmer",
"Kevtheman111", "", "Sound faceposer-syncing",
"Westwood Studios", "-Out of business-", "Sounds, spell casting concept",
"Dynamix", "http://www.dynamix.com", "Explosion sounds",
"Human Head", "http://www.runegame.com", "Effect sounds",
"Pentavision", "http://www.pentavision.co.kr", "Effect sound",
"Slyfo", "", "Wall models",
"The Darker One", "", "Gladradus and Frotchet models",
"Various mappers", "", "Creating the maps to play on!",
"The community", "", "Playing the game!"}

GM.GameType = "CTF"
GM.FlagEntity = "flag"

GM.DisabledSpells = {}

function GM:OnPlayerHitGround(pl, inwater, hitfloater, fallspeed)
	pl.NoJump = math.max(pl.NoJump or 0, CurTime() + 0.1)
	pl.DodgedInAir = nil
	pl:SetVelocityRecovery(false, true)
	local wep = pl:GetActiveWeapon()
	if wep.OnPlayerHitGround then
		wep:OnPlayerHitGround(pl, inwater, hitfloater, fallspeed)
	end

	return true
end

function GM:Move(pl, move)
	if pl:WaterLevel() == 0 and not pl:IsOnGround() then
		move:SetSideSpeed(move:GetSideSpeed() * 0.15)
		move:SetForwardSpeed(move:GetForwardSpeed() * 0.15)
	end

	if StatusWeaponHook("Move", pl, move) then return true end

	if pl:IsSprinting() then
		pl:MoveSprinting(move)
	end

	if pl.DashDodging then
		pl:SetGroundEntity(NULL)
		move:SetSideSpeed(0)
		move:SetForwardSpeed(0)
	elseif pl.WallJumping then
		pl:SetGroundEntity(NULL)
		move:SetVelocity(Vector(0,0,0))
		move:SetSideSpeed(0)
		move:SetForwardSpeed(0)
	elseif pl.Stunned then
		move:SetSideSpeed(0)
		move:SetForwardSpeed(0)
	elseif pl.WallRunning then
		move:SetMaxSpeed(0)
		move:SetMaxClientSpeed(0)
		move:SetSideSpeed(0)
		move:SetForwardSpeed(0)

		if pl.WallRunningUp then
			if CurTime() - pl.WallRunningUpStart > 0 then
				pl:SetGroundEntity(NULL)
				local spd = pl.WallVertSpeed
				if pl.Slow then spd = spd * 0.7 end
				move:SetVelocity(Vector(0, 0, spd))
			end
		else
			local spd = pl.WallRunSpeed
			if pl.Slow then spd = spd * 0.7 end
			local vNew = pl:GetForward() * spd
			vNew.z = move:GetVelocity().z + FrameTime() * 690
			move:SetVelocity(vNew)
		end
	elseif pl.HealCharge or pl.Slow then
		move:SetSideSpeed(move:GetSideSpeed() * 0.7)
		move:SetForwardSpeed(move:GetForwardSpeed() * 0.7)
	end
end

function GM:KeyPress(pl, key)
	if key == IN_FORWARD then
		pl:PressedForward()
	elseif key == IN_JUMP then
		pl:PressedJump()
	end
end

function GM:KeyRelease(pl, key)
	if key == IN_FORWARD then
		pl:ReleasedForward()
	end
end

function GM:GetRagdollEyes(pl)
	local Ragdoll = pl:GetRagdollEntity()
	if not Ragdoll then return end

	local att = Ragdoll:GetAttachment(Ragdoll:LookupAttachment("eyes"))
	if att then
		att.Pos = att.Pos + att.Ang:Forward() * -2
		att.Ang = att.Ang

		return att.Pos, att.Ang
	end
end

--[[function GM:ShouldCollide(enta, entb)
	if enta.ShouldNotCollide and enta:ShouldNotCollide(entb) or entb.ShouldNotCollide and entb:ShouldNotCollide(enta) then return false end

	return true
end]]

function GM:PlayersShouldCollide(enta, entb)
	return enta:Team() ~= entb:Team()
end

function GM:ShouldCollide(enta, entb)
	if enta.SendLua and entb.SendLua then
		return self:PlayersShouldCollide(enta, entb)
	end

	return true
end

function GM:PlayerStepSoundTime(pl, iType, bWalking)
	if iType == STEPSOUNDTIME_NORMAL or iType == STEPSOUNDTIME_WATER_FOOT then
		return 520 - pl:GetVelocity():Length()
	elseif iType == STEPSOUNDTIME_ON_LADDER then
		return 500
	end

	return 350
end

function GM:GetGameDescription()
	return self.Name
end

function GM:PhysgunPickup(pl, ent)
	return false
end

function GM:PhysgunDrop(pl, ent)
end

function GM:SetupMove(pl, move)
end

function GM:FinishMove(pl, move)
end

function GM:ContextScreenClick(aimvec, mousecode, pressed, ply)
end

function GetOpposingPlayers(iTeamID)
	local tOpposingPlayers = {}

	for _, pl in pairs(player.GetAll()) do
		local iPlayerTeam = pl:Team()
		if iPlayerTeam ~= TEAM_SPECTATE and iPlayerTeam ~= TEAM_UNASSIGNED and iPlayerTeam ~= iTeamID then
			tOpposingPlayers[#tOpposingPlayers + 1] = pl
		end
	end

	return tOpposingPlayers
end

function NormalizeAspects(tochange, lock)
	local left = ASPECTS_MAX

	if lock and ASPECTS[lock] then
		local desired = tochange[lock]
		if desired then
			desired = math.min(math.Clamp(math.ceil(tochange[lock] * 100) * 0.01, 0, 1), left)
			left = left - desired

			tochange[lock] = desired
		end
	end

	for i = 1, NUMASPECTS do
		local desired = tochange[i]
		if lock ~= i and ASPECTS[i] and desired then
			desired = math.min(math.Clamp(math.ceil(tochange[i] * 100) * 0.01, 0, 1), left)
			left = left - desired

			tochange[i] = desired
		end
	end

	for i = NUMASPECTS + 1, #tochange do
		tochange[i] = nil
	end

	return tochange
end

function IsVisible(posa, posb)
	if posa == posb then return true end
	return not util.TraceLine({start = posa, endpos = posb, mask = MASK_SOLID_BRUSHONLY}).HitWorld
end

function MeleeVisible(posa, posb, _filter)
	return not util.TraceLine({start = posa, endpos = posb, filter = _filter}).Hit
end

function TrueVisible(posa, posb)
	local filt = ents.FindByClass("projectile_*")
	filt = table.Add(filt, ents.FindByClass("prop_physics*"))
	filt = table.Add(filt, ents.FindByClass("prop_vehicle*"))
	filt = table.Add(filt, ents.FindByClass("vehicle*"))
	if GAMEMODE.FlagEntity then
		filt = table.Add(filt, ents.FindByClass(GAMEMODE.FlagEntity))
	end
	filt = table.Add(filt, player.GetAll())

	return not util.TraceLine({start = posa, endpos = posb, filter = filt, mask = MASK_SHOT}).Hit
end

function TrueVisible2(posa, posb, filtent)
	local filt = ents.FindByClass("projectile_*")
	filt = table.Add(filt, ents.FindByClass("prop_physics*"))
	filt = table.Add(filt, ents.FindByClass("prop_vehicle*"))
	filt = table.Add(filt, ents.FindByClass("vehicle*"))
	if GAMEMODE.FlagEntity then
		filt = table.Add(filt, ents.FindByClass(GAMEMODE.FlagEntity))
	end
	filt = table.Add(filt, player.GetAll())
	filt[#filt + 1] = filtent

	return not util.TraceLine({start = posa, endpos = posb, filter = filt}).Hit
end

function IsGlitched(ent, pl, passive)
	--[[local filtertab = ents.GetAll()
	for i=1, #filtertab do
		local ent = filtertab[i]

		if ent and ent:IsValid() and ent:GetClass() == "func_wall_toggle" then
			table.remove(filtertab, i)
			i = i - 1
		end
	end]]

	if util.TraceLine({start = pl:GetShootPos(), endpos = ent:GetPos(), mask=MASK_SOLID_BRUSHONLY}).Hit then
		if pl and pl.PrintMessage then
			pl:PrintMessage(HUD_PRINTTALK, "The world is between you and the prop (anti-glitch).")
		end
		return true
	end

	if util.TraceLine({start = ent:GetPos(), endpos = pl:GetShootPos(), mask=MASK_SOLID_BRUSHONLY}).Hit then
		if pl and pl.PrintMessage then
			pl:PrintMessage(HUD_PRINTTALK, "The world is between you and the prop (anti-glitch).")
		end
		return true
	end

	--[[if util.TraceLine({start = pl:GetShootPos(), endpos = ent:GetPos(), mask=COLLISION_GROUP_DEBRIS_TRIGGER}).Hit then
		if pl and pl.PrintMessage then
			pl:PrintMessage(HUD_PRINTTALK, "The world is between you and the prop (anti-glitch).")
		end
		return true
	end]]

	--[[if util.TraceLine({start = pl:GetShootPos(), endpos = ent:GetPos(), filter=filtertab}).Hit then
		if pl and pl.PrintMessage then
			pl:PrintMessage(HUD_PRINTTALK, "The team seperator walls are between you and the prop (anti-glitch).")
		end
		return true
	end]]

	local trigs = ents.FindByClass("trigger_hurt")
	table.Add(trigs, ents.FindByClass("func_disposer"))
	for _, brushent in pairs(trigs) do
		for _, findent in pairs(ents.FindInBox(brushent:LocalToWorld(brushent:OBBMins()), brushent:LocalToWorld(brushent:OBBMaxs()))) do
			if ent == findent then
				if pl and pl.PrintMessage then
					pl:PrintMessage(HUD_PRINTTALK, "You can not place props in that area.")
				end
				return true
			end
		end
	end

	return false
end

function ToMinutesSeconds(TimeInSeconds)
	local iMinutes = math.floor(TimeInSeconds / 60.0)
	return string.format("%0d:%02d", iMinutes, math.floor(TimeInSeconds - iMinutes*60))
end
