local PANEL = {}

AccessorFunc(PANEL, "strSequence", "Sequence")
AccessorFunc(PANEL, "fRotateAmount", "RotateAmount")

function PANEL:LookAtCenter()
	local aa, bb = self.Entity:OBBMins(), self.Entity:OBBMaxs()
	local len = aa:Length() * 1.5 + bb:Length() * 1.5
	self:SetCamPos(Vector(len, len, len))
	self:SetLookAt(self.Entity:OBBCenter())
end

function PANEL:SetModel(strModelName)
	if IsValid(self.Entity) then
		self.Entity:Remove()
		self.Entity = nil
	end

	if not ClientsideModel then return end

	self.Entity = ClientsideModel(strModelName, RENDER_GROUP_OPAQUE_ENTITY)
	if not IsValid(self.Entity) then return end

	self.Entity:SetNoDraw(true)

	local iSeq = self.Entity:LookupSequence(self:GetSequence() or "")
	if 0 < iSeq then self.Entity:ResetSequence(iSeq) end
end

function PANEL:LayoutEntity(Entity)
	if self.bAnimated then
		self:RunAnimation()
	end

	--Entity:SetAngles(Angle(0, RealTime() * self:GetRotateAmount(), 0))
end

function PANEL:Paint()
	if not IsValid(self.Entity) then return end

	local x, y = self:LocalToScreen(0, 0)

	self:LayoutEntity(self.Entity)

	cam.Start3D(self.vCamPos, (self.vLookatPos - self.vCamPos):Angle(), self.fFOV, x, y, self:GetWide(), self:GetTall())
	cam.IgnoreZ(true)

	render.SuppressEngineLighting(true)
	render.SetLightingOrigin(self.Entity:GetPos())
	render.ResetModelLighting(self.colAmbientLight.r/255, self.colAmbientLight.g/255, self.colAmbientLight.b/255)
	render.SetColorModulation(self.colColor.r/255, self.colColor.g/255, self.colColor.b/255)
	render.SetBlend(self.colColor.a/255)

	for i=0, 6 do
		local col = self.DirectionalLight[i]
		if col then
			render.SetModelLighting(i, col.r / 255, col.g / 255, col.b / 255)
		end
	end

	self:Draw()

	render.SuppressEngineLighting(false)
	cam.IgnoreZ(false)
	cam.End3D()

	self.LastPaint = RealTime()
end

function PANEL:Draw()
	self.Entity:DrawModel()
end

derma.DefineControl("DModelPanel2", "DModelPanel with some extra functions.", PANEL, "DModelPanel")
