local PANEL = {}

function PANEL:Init()
	self:SetPaintBackgroundEnabled(false)
	self:SetPaintBorderEnabled(false)
	self.ActiveItem = 1
	self.Items = {}
end

function PANEL:GetSpacing()
	return self.m_iSpacing or 32
end

function PANEL:SetSpacing(amount)
	self.m_iSpacing = amount or self:GetSpacing()
end

function PANEL:GetDisabled()
	return self.m_bDisabled
end

function PANEL:SetDisabled(bDisabled)
	self.m_bDisabled = bDisabled

	if bDisabled then
		self:SetAlpha(75)
		self:SetMouseInputEnabled(false)
	else
		self:SetAlpha(255)
		self:SetMouseInputEnabled(true)
	end
end

function PANEL:SetActiveItem(i)
	if self.Items[i] then
		self.ActiveItem = i
	end
end

function PANEL:GetItem(i)
	return self.Items[i]
end

function PANEL:RemoveItem(i)
	if self.Items[i] then
		self.Items[i]:Remove()
		table.remove(self.Items, i)
		self:PerformLayout()
	end
end

function PANEL:AddItem(item)
	if item then
		table.insert(self.Items, item)
		item:SetParent(self)
		local shadow = vgui.Create("NoxHorizontalMenuShadow", item)
		shadow:SetSize(item:GetWide(), item:GetTall())
		self:PerformLayout()
	end
end

function PANEL:Paint()
	derma.SkinHook("Paint", "NoxHorizontalMenu", self)

	local desireditem = self.Items[self.ActiveItem]
	local itemx, itemy = desireditem:GetPos()

	local x, y = self:GetPos()

	local desiredx = ScrW() * 0.5 - (itemx + desireditem:GetWide() * 0.5)
	if x ~= desiredx then
		self:SetPos(math.Approach(x, desiredx, FrameTime() * math.min(math.max(64, math.abs(desiredx - x)), 350)), y)
	end
end

function PANEL:PerformLayout()
	local spacing = self:GetSpacing()
	local x = 0
	local maxitems = #self.Items

	for i, item in ipairs(self.Items) do
		item:SetPos(x, 0)

		if i < maxitems then
			x = x + item:GetWide() + spacing
		else
			x = x + item:GetWide()
		end
	end

	self:SetWide(x)
	self:SetPos(ScrW() * 0.5 - wid * 0.5)
end

derma.DefineControl("DNoxHorizontalMenu", "", PANEL, "Panel")

local PANEL = {}

function PANEL:Init()
	self.Alpha = 120
end

function PANEL:Paint()
	if self:GetParent():GetParent().ActiveItem == i then
		self.Alpha = math.max(self.Alpha - FrameTime() * 350, 0)
	else
		self.Alpha = math.min(self.Alpha + FrameTime() * 200, 120)
	end

	surface.SetDrawColor(0, 0, 0, self.Alpha)
	surface.DrawRect(0, 0, self:GetWide(), self:GetTall())
end

vgui.Register("NoxHorizontalMenuShadow", PANEL, "Panel")
