pSpellsIcon = "gui/silkicons/star"
pSpellsTooltip = "Change your current profile's spell shortcuts."

function MakepSpells()
	if pSpells then return end

	local Window = vgui.Create("DPanel")
	Window:SetSize(784, 540)
	Window:SetCursor("pointer")
	pSpells = Window
end

local function ListButtonClick(btn)
	pSpells.CurrentSpellName = btn.Name
	surface.PlaySound("nox/manastunoff.wav")
	pSpells:RefreshInfo()
end

local colBack = Color(40, 40, 40, 255)
local function ippaint(self)
	draw.RoundedBox(16, 0, 0, self:GetWide(), self:GetTall(), colBack)
end

function BuildpSpells()
	local PanelList = vgui.Create("DPanelList", pSpells)
	PanelList:EnableVerticalScrollbar()
	PanelList:EnableHorizontal(false)
	local panellistwidth = 200
	PanelList:SetSize(panellistwidth, 524)
	PanelList:SetPos(8, 8)
	PanelList:SetSpacing(1)
	pSpells.SpellList = PanelList
	pSpells.CurrentSpellName = Spells[1].Name or "Magic Arrow"

	PanelList.RefreshList = function(pl)
		for _, item in pairs(pl:GetItems()) do
			pl:RemoveItem(item)
		end

		for i, spelltab in SortedPairsByMemberValue(Spells, "Name") do
			if spelltab.Mana < 1000 then
				local spellpanel = vgui.Create("DButton")

				local spellicon = vgui.Create("DImage", spellpanel)
				spellicon:SetImage(spelltab.Texture)
				spellicon:SetSize(16, 16)
				spellicon:SetPos(4, 11 - spellicon:GetTall() * 0.5)

				spellpanel:SetText(spelltab.Name)

				spellpanel:SetTall(22)
				spellpanel.Name = spelltab.Name
				spellpanel.DoClick = ListButtonClick

				PanelList:AddItem(spellpanel)
			end
		end
	end
	PanelList:RefreshList()

	pSpells.RefreshInfo = function(me)
		local spellid
		local spellname
		local spelltab
		if me.CurrentSpellName then
			spellid = NameToSpell[me.CurrentSpellName]
			spellname = me.CurrentSpellName
			spelltab = Spells[spellid]
		end

		if me.CurrentSpellInfoBox then
			me.CurrentSpellInfoBox:Remove()
			me.CurrentSpellInfoBox = nil
		end

		if spelltab then
			local infopanel = vgui.Create("DPanel", me)
			me.CurrentSpellInfoBox = infopanel
			local infow, infoh = me:GetWide() - PanelList:GetWide() - 24, me:GetTall() * 0.5 - 32
			local halfw = infow * 0.5
			infopanel:SetSize(infow, infoh)
			local x,y = PanelList:GetPos()
			infopanel:SetPos(x + PanelList:GetWide() + 8, y)
			infopanel.Paint = ippaint

			if spelltab.Texture then
				local icon = vgui.Create("DImage", infopanel)
				icon:SetImage(spelltab.Texture)
				icon:SizeToContents()
				icon:SetPos(halfw - 16, 8)
			end

			local y = 48

			local spellnamelabel = EasyLabel(infopanel, spellname, "akbarnormal")
			spellnamelabel:SetPos(halfw - spellnamelabel:GetWide() * 0.5, y)
			y = y + spellnamelabel:GetTall() + 8

			local spelldesclabel = WordBox(infopanel, spelltab.Desc)
			spelldesclabel:SetPos(halfw - spelldesclabel:GetWide() * 0.5, y)
			y = y + spelldesclabel:GetTall() + 16

			if spelltab.Mana and 0 < spelltab.Mana then
				local lbl = EasyLabel(infopanel, "Mana cost: "..spelltab.Mana, nil, COLOR_CYAN)
				lbl:SetPos(16, y)
				y = y + lbl:GetTall() + 1
			end

			if spelltab.Stamina and 0 < spelltab.Stamina then
				lbl = EasyLabel(infopanel, "Stamina cost: "..spelltab.Stamina, nil, COLOR_YELLOW)
				lbl:SetPos(16, y)
				y = y + lbl:GetTall() + 1
			end

			if spelltab.RawWords then
				lbl = EasyLabel(infopanel, "Words of Power: "..spelltab.RawWords)
				lbl:SetPos(16, y)
				y = y + lbl:GetTall() + 1
			end

			if spelltab.Delay then
				if spelltab.Delay == 0 then
					lbl = EasyLabel(infopanel, "Instantaneous casting", nil, COLOR_LIMEGREEN)
				elseif spelltab.Delay == 1 then
					lbl = EasyLabel(infopanel, "Casting time: 1 second")
				else
					lbl = EasyLabel(infopanel, "Casting time: "..spelltab.Delay.." seconds")
				end
				lbl:SetPos(16, y)
				y = y + lbl:GetTall() + 1
			end

			if spelltab.Inversable then
				lbl = EasyLabel(infopanel, "Has an alternate mode", nil, COLOR_LIMEGREEN)
				lbl:SetPos(16, y)
				y = y + lbl:GetTall() + 1
			end

			for i=1, NUMASPECTS do
				if spelltab[i] then
					lbl = EasyLabel(infopanel, "Requires ".. spelltab[i] * 100 .." "..ASPECTS[i].Name, nil, COLOR_RED)
					lbl:SetPos(16, y)
					y = y + lbl:GetTall() + 1
				end
			end
		end
	end
	pSpells:RefreshInfo()

	local posx, posy = pSpells.CurrentSpellInfoBox:GetPos()
	local dX = posx + pSpells.CurrentSpellInfoBox:GetWide() - 160
	local dY = pSpells.CurrentSpellInfoBox:GetTall() + posy + 8

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_FORWARD + IN_MOVELEFT
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 64

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_FORWARD
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 64

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_FORWARD + IN_MOVERIGHT
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dY = dY + 32
	dX = dX - 96

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_FORWARD + IN_MOVELEFT
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_FORWARD
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_FORWARD + IN_MOVERIGHT
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX - 64
	dY = dY + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_MOVELEFT
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX - 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_MOVELEFT
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 96

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_MOVERIGHT
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_MOVERIGHT
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX - 96
	dY = dY + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_BACK + IN_MOVELEFT
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_BACK
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_BACK + IN_MOVERIGHT
	button.Set = 1
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX - 96
	dY = dY + 32

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_MOVELEFT + IN_BACK
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 64

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_BACK
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	dX = dX + 64

	local button = vgui.Create("SpellBindingButton", pSpells)
	button.Bind = IN_BACK + IN_MOVERIGHT
	button.Set = 2
	button:SetPos(dX, dY)
	button:SetSize(32, 32)

	--local y = pSpells.CurrentSpellInfoBox:GetTall() + posy + 8
	local listx, listy = PanelList:GetPos()
	--local x = listx + 8 + PanelList:GetWide()

	local tentry = vgui.Create("DTextEntry", pSpells)
	tentry:SetEditable(false)
	tentry:SetMouseInputEnabled(true)
	tentry:SetMultiline(true)
	tentry:SetVerticalScrollbarEnabled(true)
	local ty = pSpells.CurrentSpellInfoBox:GetTall() + posy + 8
	tentry:SetPos(listx + 8 + PanelList:GetWide(), ty)
	tentry:SetSize(pSpells.CurrentSpellInfoBox:GetWide() - 168, pSpells:GetTall() - ty - 8)
	tentry:SetValue([[INSTRUCTIONS
	* First, click a spell on the left list to get information about it and select it as the current spell.
	* Click a spot on the grid to assign the current spell to that spot.
	* Right click a spot on the grid to clear the spell from that spot.

	SELECTING A SPELL WHILE PLAYING
	* Hold your alternate fire button (usually the right mouse button) while holding a magic weapon such as a wand or staff.
	* While still holding it down, move in the direction the spell is in on your menu. Hold alternate attack AND then hold primary attack to select spells on the outter ring of the circle.

	CASTING A SPELL WHILE PLAYING
	* Press your sprint key to cast the currently selected spell. If you have a wand then it will be cast as soon as the words of power are done. If you have a staff then you must press sprint again to discharge the spell from the staff.

	HOW TO BIND A SPELL TO A KEY
	* In your console, type:
	bind "key" "cast spell name"
	Where key is a key on your keyboard and spell name is a spell. For example, bind "g" "cast minor heal".
	* The console can be activated by pressing the (~) key on your keyboard (above TAB usually). You can enable the developer console in the advanced keyboard options of the GMod main menu.]])

	--[[local lab = EasyLabel(pSpells, "How to use...", nil, COLOR_RED)
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "1) Click a spell on the left list to get information about it.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "2) Click a spot on the grid to assign the current spell to that spot.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "* Right click a spot on the grid to clear the spell from that spot.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "To select a spell in-game...", nil, COLOR_RED)
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "1) Hold your alternate fire button with a magic weapon such as a wand or staff.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "2) Move in the direction the spell is in on your menu.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "3) Press your sprint key to cast the spell.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "* Hold alternate attack and then hold primary attack to select spells on the outside of the circle.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "To bind a specific key to cast a specific spell...", nil, COLOR_RED)
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "In your console, type bind \"key\" \"cast spell name\"", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "Where key is a key on your keyboard and spell name is a spell.", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "For example, bind \"g\" \"cast minor heal\".", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "The console can be activated by pressing the (~) key on your keyboard (above TAB usually).", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1

	local lab = EasyLabel(pSpells, "You can enable the developer console in the advanced keyboard options of the GMod (hit ESC).", "DefaultSmall")
	lab:SetPos(x, y)
	y = y + lab:GetTall() + 1]]
end

local CurrentSpellButton

function oldMakepSpells()
	local label = vgui.Create("DLabel", Window)
	local txt = "Click on a spell up top and then click on a slot to assign it to a speedkey."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth

	local label = vgui.Create("DLabel", Window)
	local txt = "Right click a speedkey to clear it."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth

	local label = vgui.Create("DLabel", Window)
	local txt = "Spells set configs are saved on your computer by class name."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth

	local label = vgui.Create("DLabel", Window)
	local txt = "Some classes do not use spells and this menu will be blank if so."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth

	local label = vgui.Create("DLabel", Window)
	local txt = "  - Hold right mouse button and move in that direction for spell set 1."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth

	local label = vgui.Create("DLabel", Window)
	local txt = "  - Hold right AND left mouse button and move in that direction for spell set 2."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth

	local label = vgui.Create("DLabel", Window)
	local txt = "  - Cast your active spell by pushing your sprint key."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")

	y = y + geth * 2

	local label = vgui.Create("DLabel", Window)
	local txt = "You can bind spells directly to keys by using bind \"t\" \"cast death ray\" for example."
	label:SetPos(x, y)
	label:SetSize(surface.GetTextSize(txt))
	label:SetTextColor(color_white)
	label:SetText(txt)
	label:SetFont("Default")
end

local PANEL = {}
function PANEL:OnMousePressed(mc)
	if not pSpells.CurrentSpellName and mc ~= MOUSE_RIGHT then return end

	local set = self.Set
	local bind = self.Bind

	if mc == MOUSE_LEFT then
		MENUPROFILE.SpellSheet[bind] = MENUPROFILE.SpellSheet[bind] or {}
		MENUPROFILE.SpellSheet[bind][set] = pSpells.CurrentSpellName or "-"
		surface.PlaySound("nox/managain.wav")
		if ACTIVEPROFILE == ASPECTFILE then
			SPELL_SHEET = MENUPROFILE.SpellSheet
		end
		SaveProfile(MENUPROFILE)
	elseif mc == MOUSE_RIGHT then
		MENUPROFILE.SpellSheet[bind] = MENUPROFILE.SpellSheet[bind] or {}
		MENUPROFILE.SpellSheet[bind][set] = "-"
		surface.PlaySound("nox/manastuncast.wav")
		if ACTIVEPROFILE == ASPECTFILE then
			SPELL_SHEET = MENUPROFILE.SpellSheet
		end
		SaveProfile(MENUPROFILE)
	end
end
function PANEL:Paint()
	local tab = NameToSpellLower[ string.lower(tostring((MENUPROFILE.SpellSheet[self.Bind] or {})[self.Set])) ]
	if tab then
		surface.SetTexture(Spells[tab].Icon)
		surface.SetDrawColor(255, 255, 255, 255)
		surface.DrawTexturedRect(0, 0, self:GetWide(), self:GetTall())
	else
		surface.SetDrawColor(255, 255, 255, 255)
		surface.DrawOutlinedRect(0, 0, self:GetWide(), self:GetTall())
	end

	return true
end
vgui.Register("SpellBindingButton", PANEL, "Button")
