local meta = FindMetaTable("Entity")
if not meta then return end

function meta:SetMass(mass)
	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		phys:SetMass(mass)
	end
end

function meta:GetMass()
	local phys = self:GetPhysicsObject()
	if phys:IsValid() then
		return phys:GetMass()
	end

	return 1
end

-- I don't know why but using pl:LocalToWorld(pl:OBBCenter()) will fuck up drawing for that frame if parented. It'll appear to be 'attached' to the player's head.
-- Using this method doesn't do it.
function meta:CenterOnPlayer(pl)
	self:SetPos(pl:GetPos() + Vector(0, 0, pl:OBBMaxs().z * 0.5))
end

function meta:Thought(text)
	self:Talk("> "..text, self)
end

function meta:ThoughtAndFloatie(text, colid)
	self:Thought(text)
	self:Floatie(text, colid)
end

function meta:RPGName(viewer)
	local itemdata = self:GetItem() or self:GetDefaultItemData()
	if itemdata then
		return util.NameByAmount(itemdata.Name, self:GetAmount())
	end
end

function meta:TakeNonLethalDamage(damage, damagetype, attacker, inflictor)
	if self:IsPlayer() then
		self:SetHealth(self:Health() + 1000)
		self:TakeSpecialDamage(damage, damagetype, attacker, inflictor)
		local vel = self:GetVelocity()
		self:SetHealth(math.max(1, self:Health() - 1000))
		self:SetVelocity(self:GetVelocity() * -1 + vel)
		if self:Health() == 1 and (self.m_NonLethalImmunity or 0) <= CurTime() then
			self:Stun(6)
			self:KnockDown(6)
			self.m_NonLethalImmunity = CurTime() + 10
		end
	else
		self:TakeSpecialDamage(damage, damagetype, attacker, inflictor)
	end
end

function meta:IsInteractable()
	return self:IsCharacter() and self:Alive() or self:GetMoveType() ~= MOVETYPE_NONE and self:GetMoveType() ~= MOVETYPE_OBSERVER
end

function meta:GetTalkFilter()
	local filter = RecipientFilter()

	for _, pl in pairs(self:GetTalkRecipients()) do
		filter:AddPlayer(pl)
	end

	return filter
end

function meta:GetTalkRecipients(radius)
	local tab = {}

	local mouthpos = self:MouthPos()

	for _, pl in pairs(player.GetAll()) do
		if pl:CanUnderstand(self) and (pl == self or pl:NewEyePos():Distance(mouthpos) <= (radius or TALK_RADIUS)) then
			tab[#tab + 1] = pl
		end
	end

	return tab
end

function meta:GetItemByUIDOrDataName(argument)
	local itemid = tonumber(argument)
	return itemid and Items[itemid] or self:GetContainer():GetItemNonStrict(argument)
end

function meta:GetUsableItemByUIDOrDataName(argument)
	local item = self:GetItemByUIDOrDataName(argument)
	if item and item:IsUsableBy(self) then return item end
end

function meta:SetRemoving(removing)
	self.Removing = removing
end

function meta:GetRemoving()
	return self.Removing
end
meta.IsRemoving = meta.GetRemoving

meta.OldRemove = meta.Remove
function meta:Remove()
	self:SetRemoving(true)
	self:OldRemove()
end

function meta:GetAttachmentByName(name)
	local attachid = self:LookupAttachment(name)
	if attachid ~= 0 then
		return self:GetAttachment(attachid)
	end
end
meta.GetNamedAttachment = meta.GetAttachmentByName

if SERVER then
	function meta:GetItem()
		return self.ItemData or Items[self:GetItemUID()]
	end

	function meta:SetItemUID(uid)
		self:SetDTInt(2, uid)
	end
end

if CLIENT then
	function meta:GetItem()
		return Items[self:GetItemUID()] or self.ItemData
	end

	function meta:SetItemUID(uid)
	end
end
meta.GetContainer = meta.GetItem

function meta:SetItem(item)
	self.ItemData = item
	if item then
		self:SetItemUID(item.UID)
	else
		self:SetItemUID(-1)
	end
end

function meta:GetItemUID()
	return self:GetDTInt(2)
end

meta.SetContainer = meta.SetItem

function meta:Alive()
	return 0 < self:Health() 
end

function meta:IsInWater()
	return self:WaterLevel() > 0
end

function meta:IsSwimming()
	return self:WaterLevel() >= 3
end

function meta:IsSubmerged()
	return self:WaterLevel() >= 4
end

function meta:IsProjectile()
	return self:GetCollisionGroup() == COLLISION_GROUP_PROJECTILE --string.sub(self:GetClass(), 1, 11) == "projectile_"
end

function meta:CallCastSpellEnchantments()
end

function meta:GetHeadingTo(target)
	return (target:LocalToWorld(target:OBBCenter()) - self:LocalToWorld(self:OBBCenter())):Normalize()
end

function meta:CapSkillLevel(skill)
	self:SetSkillLevel(math.max(self:GetSkillLevel(), skill))
end

function meta:GetAmount()
	return math.max(1, self:GetDTInt(3))
end

function meta:SetAmount(amount)
	self:SetDTInt(3, amount)
end

function meta:IsMonster()
	return false
end

function meta:GetNameColor(viewer)
	return COLOR_LIGHTBLUE
end

function meta:TakeSpecialDamage(damage, damagetype, attacker, inflictor, hitpos)
	attacker = attacker or self
	if not attacker:IsValid() then attacker = self end
	inflictor = inflictor or attacker
	if not inflictor:IsValid() then inflictor = attacker end

	local dmginfo = DamageInfo()
	dmginfo:SetDamage(damage)
	dmginfo:SetAttacker(attacker)
	dmginfo:SetInflictor(inflictor)
	dmginfo:SetDamagePosition(hitpos or self:NearestPoint(inflictor:NearestPoint(self:LocalToWorld(self:OBBCenter()))))
	dmginfo:SetDamageType(damagetype)
	self:TakeDamageInfo(dmginfo)
end

function meta:ThrowFromPosition(pos, force)
	if force == 0 or self:IsProjectile() then return end

	if self:IsPlayer() then
		stat.Start(force)
			self:StatusWeaponHook("AlterThrownForce", force)
		force = stat.End()
	end

	if self:GetMoveType() == MOVETYPE_VPHYSICS then
		local phys = self:GetPhysicsObject()
		if phys:IsValid() and phys:IsMoveable() then
			local nearest = self:NearestPoint(pos)
			phys:ApplyForceOffset(force * 50 * (nearest - pos):Normalize(), nearest)
		end
	elseif self:GetMoveType() >= MOVETYPE_WALK and self:GetMoveType() < MOVETYPE_PUSH then
		self:SetGroundEntity(NULL)
		if self.KnockDown and FORCE_KNOCKDOWN <= math.abs(force) then
			self:KnockDown()
		end
		self:SetVelocity(force * (self:LocalToWorld(self:OBBCenter()) - pos):Normalize())
	end
end

function meta:SetTeamID(int)
	self.TeamID = int
end

function meta:GetTeamID()
	return self.TeamID or 0
end

function meta:IsCharacter()
	return self:IsPlayer() or self:IsNPC()
end

function meta:IsPlayerCharacter()
	return self:IsPlayer() --return self:IsCharacter() and self:GetTeamID() == TEAM_HUMAN
end

function meta:IsInBox(vecmin, vecmax)
	for _, ent in pairs(ents.FindInBox(vecmin, vecmax)) do
		if self == ent then return true end
	end

	return false
end

function meta:SetAlpha(a)
	local r, g, b, _ = self:GetColor()
	self:SetColor(r, g, b, a)
end

function meta:SetAlphaModulation(a)
	self:SetAlpha(a * 255)
end

function meta:GetAlpha()
	local r,g,b,a = self:GetColor()
	return a
end
meta.GetVisibility = meta.GetAlpha

function meta:GetAlphaModulation()
	return self:GetAlpha() / 255
end

function meta:IsOpaque()
	return self:GetAlpha() == 255
end

function meta:IsInvisible()
	return self:GetAlpha() == 0
end

function meta:GetItemClass()
	return string.sub(self:GetClass(), 6)
end

function meta:GetDefaultItemData()
	return GetItemData(self:GetItemClass())
end

function meta:GetZone()
	return self.m_Zone or "0"
end

function meta:SetZone(zoneid)
	self.m_Zone = zoneid
end

function meta:ClearZone()
	self:SetZone(nil)
end

function meta:AddGold(...)
	return self:AddItem("gold_coin", ...)
end
meta.GiveGold = meta.AddGold

function meta:RemoveGold(...)
	return self:RemoveItem("gold_coin", ...)
end
meta.TakeGold = meta.RemoveGold
meta.DestroyGold = meta.RemoveGold

function meta:HasGold(...)
	return self:HasItem("gold_coin", ...)
end

function meta:GetGold()
	return self:GetItemAmount("gold_coin")
end

function meta:AddItem(...)
	return self:GetContainer():AddItemNonStrict(...)
end
meta.GiveItem = meta.AddItem

function meta:RemoveItem(...)
	return self:GetContainer():RemoveItemNonStrict(...)
end
meta.TakeItem = meta.RemoveItem
meta.DestroyItem = meta.RemoveItem

function meta:HasItem(...)
	return self:GetContainer():HasItemNonStrict(...)
end

function meta:GetItemAmount(...)
	return self:GetContainer():GetItemAmountNonStrict(...)
end
meta.ItemAmount = meta.GetItemAmount

function meta:AddOnOutput(key, value)
	self[key] = self[key] or {}
	local tab = string.Explode(",", value)
	table.insert(self[key], {entityname = tab[1], input = tab[2], args = tab[3], delay = tab[4], reps = tab[5]})
end

local function DelayInput(ent, input, activator, caller, args)
	if ent:IsValid() then
		ent:Input(input, activator, caller, args)
	end
end
function meta:FireOutput(outpt, activator, caller)
	local intab = self[outpt]
	if intab then
		for key, tab in pairs(intab) do
			for __, subent in pairs(ents.FindByName(tab.entityname)) do
				if tab.delay == 0 then
					subent:Input(tab.input, activator, caller, tab.args)
				else
					timer.Simple(tab.delay, DelayInput, subent, tab.input, activator, caller, tab.args)
				end
			end
		end
	end
end

function meta:MouthPos()
	return self:NewEyePos() + self:GetUp() * -3
end
meta.NewEyePos = meta.EyePos
