include("global/noxiousdb.lua")

AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")
AddCSLuaFile("obj_player_extend.lua")

include("shared.lua")

function gmod.BroadcastLua(lua)
	for _, pl in pairs(player.GetAll()) do
		pl:SendLua(lua)
	end
end

--[[function GM:SetPlayerAnimation(pl, anim)
end]]

function GM:Initialize()
	SetGlobalFloat("roundstarttime", 10)
end

function GM:InitPostEntity()
end

function GM:PlayerLoadout(pl)
end

function GM:PlayerShouldTakeDamage(pl, attacker)
	return false
end

function GM:PlayerInitialSpawn(pl)
	pl:SetTeam(TEAM_SPECTATING)
end

function GM:MakeCharacter(pl)
	if self:GetGameState() ~= 1 then return end

	local ent = ents.Create("prop_ragdoll")
	if ent:IsValid() then
		local pos = pl:GetPos() + Vector(0,0,64)
		ent:SetModel(pl.Class.Model)
		ent:SetAngles(pl:GetAngles())
		ent:SetPos(pos)
		ent:Spawn()
		ent.Player = pl

		local VecOffset = pos - ent:GetPos()
		for i=0, ent:GetPhysicsObjectCount() - 1 do
			local phys = ent:GetPhysicsObjectNum(i)
			phys:SetPos(phys:GetPos() + VecOffset)
			phys:EnableDrag(false)
		end

		pl.Ragdoll = ent
		pl:Spectate(OBS_MODE_ROAM)
		//pl:SpectateEntity(ent)
		timer.Simple(0.5, pl.SetRagdoll, pl, ent)

		pl.Class.Initialize(ent, pl)

		pl:SetHealth(1)
		ent:SetNetworkedEntity("player", pl)
	end
end

function GM:PlayerSpawn(pl)
	//pl:UnSpectate()

	local gamestate = self:GetGameState()

	pl:SetNoDraw(true)

	if pl.Ragdoll and pl.Ragdoll:IsValid() then
		pl.Ragdoll:Remove()
	end

	pl.Class = Classes["Robert Loggia"]
	pl:SetNetworkedString("class", "Robert Loggia")

	if gamestate == 1 and pl:Team() == TEAM_FIGHTING then
		self:MakeCharacter(pl)
	else
		pl:Spectate(OBS_MODE_ROAM)
	end

	pl:SetGroundEntity(nil)
	pl:SetGravity(0)
	//pl:SetMoveType(MOVETYPE_NONE)
	pl:SetCollisionGroup(COLLISION_GROUP_WORLD)
end

function GM:EndRound(winner)
	if winner then
		winner:AddFrags(1)
		winner:SetTeam(TEAM_SPECTATING)
		gmod.BroadcastLua("GAMEMODE:EndRound("..winner:EntIndex()..")")
		if winner.Ragdoll and winner.Ragdoll:IsValid() then
			winner.Ragdoll:Remove()
		end
	end
	self:SetGameState(2)
end

function GM:DoPlayerDeath(pl)
	if pl.Ragdoll and pl.Ragdoll:IsValid() then
		pl.Ragdoll:Remove()
		pl:SetDeaths(pl:Deaths() - 1)
		if pl:Deaths() <= 0 then
			pl:SetTeam(TEAM_SPECTATING)
			local curfighting = team.GetPlayers(TEAM_FIGHTING)
			if #curfighting <= 1 then
				self:EndRound(curfighting[1])
			end
		end
	end
end

function GM:PlayerDisconnected(pl)
	if pl.Ragdoll and pl.Ragdoll:IsValid() then
		pl.Ragdoll:Remove()
		local curfighting = team.GetPlayers(TEAM_FIGHTING)
		if #curfighting <= 0 then
			self:EndRound()
		elseif #curfighting == 1 then
			self:EndRound(curfighting[1])
		end
	end
end

function GM:PlayerDeathSound()
	return true
end

function GM:CanPlayerSuicide(pl)
	return true
end

function GM:PlayerDeathThink(pl)
	if pl.NextSpawnTime and pl.NextSpawnTime > CurTime() then return end

	pl:Spawn()
end

function GM:KeyPress(pl, key)
	if pl.Ragdoll and pl.Ragdoll:IsValid() then
		pl.Class.KeyPress(key, pl, pl.Ragdoll)
	end
end

function GM:Think()
	local gamestate = self:GetGameState()

	if gamestate == 0 then
		if CurTime() >= GetGlobalFloat("roundstarttime") then
			if team.NumPlayers(TEAM_SPECTATING) < 2 then
				SetGlobalFloat("roundstarttime", CurTime() + 30)
			else
				self:SetGameState(1)
			end
		end
	elseif gamestate == 1 then
		for _, pl in pairs(player.GetAll()) do
			if pl.Ragdoll and pl.Ragdoll:IsValid() then
				pl.Class.Think(pl.Ragdoll, pl)
			end
		end

		if CurTime() >= GetGlobalFloat("suddendeathtime") then
			SetGlobalFloat("suddendeathtime", 999999)
			for _, pl in pairs(team.GetPlayers(TEAM_FIGHTING)) do
				pl:Spawn()
				pl:SetHealth(300)
				pl:SetDeaths(1)
				pl:PrintMessage(4, "Sudden Death!")
			end
		end
	else
		if CurTime() >= GetGlobalFloat("preroundstarttime") then
			self:SetGameState(0)
		end
	end
end

function GM:SetGameState(int)
	SetGlobalInt("gamestate", int)

	if int == 0 then // Pre-round
		for _, pl in pairs(player.GetAll()) do
			//pl.Class = nil
			pl:SetTeam(TEAM_SPECTATING)
		end
		SetGlobalFloat("roundstarttime", CurTime() + 30)
	elseif int == 1 then // Playing
		SetGlobalFloat("suddendeathtime", CurTime() + 300)
		for _, pl in pairs(player.GetAll()) do
			if pl.Class then // They decided to join
				pl:SetTeam(TEAM_FIGHTING)
				pl:SetDeaths(3)
				pl:Spawn()
			end
		end
	elseif int == 2 then // Post-round
		SetGlobalFloat("preroundstarttime", CurTime() + 10)
		/*for _, pl in pairs(player.GetAll()) do
			pl.Class = nil
			pl:SetNetworkedString("class", "")
		end*/
	end

	gmod.BroadcastLua("GAMEMODE:SetGameState("..int..")")
end

local function ChooseClass(pl, command, arguments)
	if GAMEMODE:GetGameState() ~= 1 then return end
	if not arguments[1] then return end

	arguments = table.concat(arguments, " ")

	if not Classes[arguments] then return end

	pl.Class = Classes[arguments]
	pl:SetNetworkedString("class", arguments)
end
concommand.Add("chooseclass", ChooseClass)
