AddCSLuaFile("cl_ctf.lua")

function GM:CTFInitialize()
	self.Name = self.Name.." (CTF)"
	self.GameType = "CTF"
	self.FlagEntity = "flag"
	gmod.BroadcastLua("GAMEMODE:CTFInitialize()")
	hook.Add("PlayerReady", "GMSend2", function(pl)
		pl:SendLua("GAMEMODE:CTFInitialize()")
	end)

	for _, ent in pairs(ents.FindByClass("logic_gametypeinitialize")) do
		if ent.GameType == self.GameType then
			ent:FireOutput("OnInitialize", ent, ent)
		end
	end

	for i=1, #TEAMS_PLAYING do
		local teamid = TEAMS_PLAYING[i]
		if team.TeamInfo[ teamid ].FlagPoint then
			local ent = ents.Create(self.FlagEntity)
			if ent:IsValid() then
				ent:SetPos(team.TeamInfo[teamid].FlagPoint + Vector(0, 0, -16))
				ent:SetTeamID(teamid)
				ent:Spawn()
				team.TeamInfo[teamid].Flag = ent
			end
		end
	end

	function self:HandleOvertime()
		local topscore = 0
		local topteam = 0
		for _, teamid in pairs(TEAMS_PLAYING) do
			if topscore < team.GetScore(teamid) then
				topscore = team.GetScore(teamid)
				topteam = teamid
			end
		end
		local numwin = 0
		for _, teamid in pairs(TEAMS_PLAYING) do
			if topscore == team.GetScore(teamid) then
				numwin = numwin + 1
			end
		end

		if numwin == 1 then
			self:EndGame(topteam, {})
		else
			self:EndGame(0, {})
		end
	end

	function self:FlagTaken(selfteam, byplayer)
		umsg.Start("FTak")
			umsg.Short(selfteam)
			umsg.String(byplayer:Name())
		umsg.End()
	end

	function self:FlagDropped(selfteam, byplayer)
		umsg.Start("FDro")
			umsg.Short(selfteam)
			if type(byplayer) == "string" then
				umsg.String(byplayer)
			else
				umsg.String(byplayer:Name())
			end
		umsg.End()
	end

	function self:FlagReturned(pl, myteam, faraway)
		if not pl then
			umsg.Start("FRet")
				umsg.String("Auto-return")
				umsg.Short(myteam)
				umsg.Bool(faraway)
			umsg.End()
			return
		end

		if faraway then
			pl:AddMoney(BONUS_RETURN)
			pl:AddFrags(3)
			pl:AddDefense(3)
		end

		umsg.Start("FRet")
			umsg.String(pl:Name())
			umsg.Short(myteam)
			umsg.Bool(faraway)
		umsg.End()
	end

	local CTFCaptureAwards = {}
	CTFCaptureAwards[50] = "50_caps"
	CTFCaptureAwards[125] = "125_caps"
	CTFCaptureAwards[250] = "250_caps"
	CTFCaptureAwards[500] = "500_CTF_Captures"
	CTFCaptureAwards[750] = "750_CTF_Captures"
	CTFCaptureAwards[1000] = "1000_CTF_Captures"

	function self:FlagCaptured(carrier, selfteam, otherteam)
		if ENDGAME then return end

		team.AddScore(selfteam, 1)

		--[[local carriers = team.TeamInfo[otherteam].Flag.Carriers
		local mostcarried = 0
		local mostcarriedby = carrier
		for seperatecarrier, timecarried in pairs(carriers) do
			if mostcarried < timecarried then
				mostcarried = timecarried
				mostcarriedby = seperatecarrier
			end
		end]]

		carrier:AddFrags(15)
		carrier:AddMoney(BONUS_CAPTURE)
		-- TODO: Change to new stats
		--[[carrier.CTFCaptures = carrier.CTFCaptures + 1

		if CTFCaptureAwards[carrier.CTFCaptures] then
			NDB.GiveAward(carrier, CTFCaptureAwards[carrier.CTFCaptures])
		end]]

		umsg.Start("FCap")
			umsg.String(carrier:Name())
			umsg.Short(selfteam)
			umsg.Short(otherteam)
		umsg.End()

		if CurTime() <= NEXT_FIGHTMODE + 30 and not carrier:HasAward("Stealing_First") then
			carrier:GiveAward("Stealing_First")
		end

		carrier.CapturesThisRound = (carrier.CapturesThisRound or 0) + 1

		if MAX_SCORE <= team.GetScore(selfteam) then
			self:EndGame(selfteam, {})
			if carrier.CapturesThisRound == MAX_SCORE and not carrier:HasAward("Greedy") then
				carrier:GiveAward("Greedy")
			end
		else
			gamemode.Call("StartIntermission", carrier)
			timer.Simple(BLITZ_INTERMISSION_TIME - 0.125, gamemode.Call, "CleanUpForNewRound")
			timer.Simple(BLITZ_INTERMISSION_TIME, gamemode.Call, "EndIntermission")
		end

		local pos = team.TeamInfo[selfteam].FlagPoint
		if pos then
			local ang = Angle(0, 0, 0)
			ParticleEffect("citadel_shockwave", pos, ang)
			ParticleEffect("aurora_shockwave", pos, ang)
		end
	end

	function SortTeams(a, b)
		local aamount = team.NumPlayers(a)
		local bamount = team.NumPlayers(b)

		if aamount < bamount * 1.2 and bamount * 0.8 < aamount then
			if BUILD_LENGTH < CurTime() then
				local ascore = team.GetScore(a)
				local bscore = team.GetScore(b)
				if ascore == bscore then
					return team.TotalFrags(a) < team.TotalFrags(b)
				else
					return ascore < bscore
				end
			else
				return team.TotalWeight(a) < team.TotalWeight(b)
			end
		end

		return aamount < bamount
	end
end
